@@ -1,5 +1,4 @@
-{-# LANGUAGE CPP, MultiParamTypeClasses #-}
-{-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
 module Darcs.Test.Patch.Arbitrary.PrimV1 where
 
 import qualified Darcs.Test.Patch.Arbitrary.Generic as T
@@ -20,39 +19,44 @@ import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Unsafe
 import Darcs.Patch.Witnesses.Ordered
-import Darcs.Patch.Prim.V1 ()
-import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk, TokReplace ), Prim( FP ), isIdentity )
+import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk, TokReplace ), isIdentity )
+import qualified Darcs.Patch.Prim.V1.Core as Prim ( Prim( FP ) )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
+import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
 import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.FileHunk( IsHunk( isHunk ), FileHunk(..) )
 
 import Darcs.Test.Patch.V1Model
 import Darcs.Util.Path
+import qualified Darcs.Util.Tree as UT ( Tree )
 import Darcs.Test.Util.QuickCheck ( alpha, notIn, maybeOf )
 
 import Darcs.UI.Commands.Replace ( defaultToks )
 import Darcs.Patch.Prim
+import Darcs.Patch.Apply ( ApplyState )
 
 import qualified Data.ByteString.Char8 as BC
 import Data.Maybe ( isJust )
 
-#include "impossible.h"
+type Prim1 = V1.Prim
+type Prim2 = V2.Prim
 
-patchFromTree :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . p wY wZ -> t) -> WithStartState V1Model (Tree Prim) wX -> t
+patchFromTree :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . p wY wZ -> t) -> WithStartState V1Model (Tree prim) wX -> t
 patchFromTree = T.patchFromTree
 
-mergePairFromTree :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . (p :\/: p) wY wZ -> t) -> WithStartState V1Model (Tree Prim) wX -> t
+mergePairFromTree :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . (p :\/: p) wY wZ -> t) -> WithStartState V1Model (Tree prim) wX -> t
 mergePairFromTree = T.mergePairFromTree
 
-mergePairFromTWFP :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . (p :\/: p) wY wZ -> t) -> WithStartState V1Model (TreeWithFlattenPos Prim) wX -> t
+mergePairFromTWFP :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . (p :\/: p) wY wZ -> t) -> WithStartState V1Model (TreeWithFlattenPos prim) wX -> t
 mergePairFromTWFP = T.mergePairFromTWFP
 
-commutePairFromTWFP :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . (p :> p) wY wZ -> t) -> WithStartState V1Model (TreeWithFlattenPos Prim) wX -> t
+commutePairFromTWFP :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . (p :> p) wY wZ -> t) -> WithStartState V1Model (TreeWithFlattenPos prim) wX -> t
 commutePairFromTWFP = T.commutePairFromTWFP
 
-commutePairFromTree :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . (p :> p) wY wZ -> t) -> WithStartState V1Model (Tree Prim) wX -> t
+commutePairFromTree :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . (p :> p) wY wZ -> t) -> WithStartState V1Model (Tree prim) wX -> t
 commutePairFromTree = T.commutePairFromTree
 
-commuteTripleFromTree :: (RepoPatch p, PrimOf p ~ Prim) => (forall wY wZ . (p :> p :> p) wY wZ -> t) -> WithStartState V1Model (Tree Prim) wX -> t
+commuteTripleFromTree :: (RepoPatch p, PrimOf p ~ prim) => (forall wY wZ . (p :> p :> p) wY wZ -> t) -> WithStartState V1Model (Tree prim) wX -> t
 commuteTripleFromTree = T.commuteTripleFromTree
 
 nonEmptyHunk :: (IsHunk p) => p wX wY -> Bool
@@ -69,25 +73,41 @@ nonEmptyHunksTriple (p1 :> p2 :> p3) = nonEmptyHunk p1 && nonEmptyHunk p2 && non
 nonEmptyHunksFLPair :: (IsHunk p) => (FL p :> FL p) wX wY -> Bool
 nonEmptyHunksFLPair (ps :> qs) = allFL nonEmptyHunk ps && allFL nonEmptyHunk qs
 
-type instance ModelOf Prim = V1Model
-instance ArbitraryPrim Prim
+type instance ModelOf Prim1 = V1Model
+type instance ModelOf Prim2 = V1Model
+instance ArbitraryPrim Prim1
+instance ArbitraryPrim Prim2
 
-instance NullPatch Prim where
-  nullPatch (FP _ fp) = nullPatch fp
-  nullPatch p | IsEq <- isIdentity p = IsEq
+instance NullPatch Prim2 where
+  nullPatch (V2.Prim (Prim.FP _ fp)) = nullPatch fp
+  nullPatch p | IsEq <- isIdentity (V2.unPrim p) = IsEq
+  nullPatch _ = NotEq
+
+instance NullPatch Prim1 where
+  nullPatch (V1.Prim (Prim.FP _ fp)) = nullPatch fp
+  nullPatch p | IsEq <- isIdentity (V1.unPrim p) = IsEq
   nullPatch _ = NotEq
 
 instance NullPatch FilePatchType where
   nullPatch (Hunk _ [] []) = unsafeCoerceP IsEq -- is this safe?
   nullPatch _ = NotEq
 
-instance MightBeEmptyHunk Prim where
-  isEmptyHunk (FP _ (Hunk _ [] [])) = True
+instance MightBeEmptyHunk Prim1 where
+  isEmptyHunk (V1.Prim (Prim.FP _ (Hunk _ [] []))) = True
   isEmptyHunk _ = False
 
-instance MightHaveDuplicate Prim
+instance MightBeEmptyHunk Prim2 where
+  isEmptyHunk (V2.Prim (Prim.FP _ (Hunk _ [] []))) = True
+  isEmptyHunk _ = False
 
-instance Arbitrary (Sealed2 (FL (WithState V1Model Prim))) where
+instance MightHaveDuplicate Prim1
+instance MightHaveDuplicate Prim2
+
+instance Arbitrary (Sealed2 (FL (WithState V1Model Prim1))) where
+  arbitrary = do repo <- ourSmallRepo
+                 liftM (unseal (seal2 . wesPatch)) $ arbitraryState repo
+
+instance Arbitrary (Sealed2 (FL (WithState V1Model Prim2))) where
   arbitrary = do repo <- ourSmallRepo
                  liftM (unseal (seal2 . wesPatch)) $ arbitraryState repo
 
@@ -95,7 +115,7 @@ instance Arbitrary (Sealed2 (FL (WithState V1Model Prim))) where
 --   showDict1 = ShowDictClass
 
 -- WithState and propFail are handy for debugging arbitrary code
-propFail :: Int -> Tree Prim wX -> Bool
+propFail :: Int -> Tree prim wX -> Bool
 propFail n xs = sizeTree xs < n
 
 ----------------------------------------------------------------------
@@ -153,21 +173,21 @@ aTokReplace content
 ----------------------------------------------------------------------
 -- ** Prim generators
 
-aHunkP :: forall wX wY . (AnchoredPath,File) -> Gen (Prim wX wY)
+aHunkP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen (prim wX wY)
 aHunkP (path,file)
   = do Hunk pos old new <- aHunk content
        return $ hunk (ap2fp path) pos old new
   where
       content = fileContent file
 
-aTokReplaceP :: forall wX wY . (AnchoredPath,File) -> Gen (Prim wX wY)
+aTokReplaceP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen (prim wX wY)
 aTokReplaceP (path,file)
   = do TokReplace tokchars old new <- aTokReplace content
        return $ tokreplace (ap2fp path) tokchars old new
   where
       content = fileContent file
 
-anAddFileP :: forall wX wY . (AnchoredPath,Dir) -> Gen (Prim wX wY)
+anAddFileP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,Dir) -> Gen (prim wX wY)
 anAddFileP (path,dir)
   = do newFilename <- aFilename `notIn` existing
        let newPath = path `appendPath` newFilename
@@ -175,11 +195,11 @@ anAddFileP (path,dir)
   where
       existing = map fst $ filterFiles $ dirContent dir
 
-aRmFileP :: forall wX wY . AnchoredPath   -- ^ Path of an empty file
-                          -> Prim wX wY
+aRmFileP :: forall prim wX wY . PrimPatch prim => AnchoredPath   -- ^ Path of an empty file
+                          -> prim wX wY
 aRmFileP path = rmfile (ap2fp path)
 
-anAddDirP :: forall wX wY . (AnchoredPath,Dir) -> Gen (Prim wX wY)
+anAddDirP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,Dir) -> Gen (prim wX wY)
 anAddDirP (path,dir)
   = do newDirname <- aDirname `notIn` existing
        let newPath = path `appendPath` newDirname
@@ -187,11 +207,11 @@ anAddDirP (path,dir)
   where
       existing = map fst $ filterDirs $ dirContent dir
 
-aRmDirP :: forall wX wY . AnchoredPath    -- ^ Path of an empty directory
-                        -> Prim wX wY
+aRmDirP :: forall prim wX wY . PrimPatch prim => AnchoredPath    -- ^ Path of an empty directory
+                        -> prim wX wY
 aRmDirP path = rmdir (ap2fp path)
 
-aMoveP :: forall wX wY . Gen Name -> AnchoredPath -> (AnchoredPath,Dir) -> Gen (Prim wX wY)
+aMoveP :: forall prim wX wY . PrimPatch prim => Gen Name -> AnchoredPath -> (AnchoredPath,Dir) -> Gen (prim wX wY)
 aMoveP nameGen oldPath (dirPath,dir)
   = do newName <- nameGen `notIn` existing
        let newPath = dirPath `appendPath` newName
@@ -199,8 +219,9 @@ aMoveP nameGen oldPath (dirPath,dir)
   where
       existing = map fst $ dirContent dir
 
--- | Generates any type of 'Prim' patch, except binary and setpref patches.
-aPrim :: forall wX wY . V1Model wX -> Gen (WithEndState V1Model (Prim wX) wY)
+-- | Generates any type of 'prim' patch, except binary and setpref patches.
+aPrim :: forall prim wX wY . (PrimPatch prim, ApplyState prim ~ RepoState V1Model)
+      => V1Model wX -> Gen (WithEndState V1Model (prim wX) wY)
 aPrim repo
   = do mbFile <- maybeOf repoFiles
        mbEmptyFile <- maybeOf $ filter (isEmpty . snd) repoFiles
@@ -266,7 +287,7 @@ aPrim repo
 -- *** Pairs of primitive patches
 
 -- Try to generate commutable pairs of hunks
-hunkPairP :: forall wX wY . (AnchoredPath,File) -> Gen ((Prim :> Prim) wX wY)
+hunkPairP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen ((prim :> prim) wX wY)
 hunkPairP (path,file)
   = do h1@(Hunk l1 old1 new1) <- aHunk content
        (delta, content') <- selectChunk h1 content
@@ -284,7 +305,7 @@ hunkPairP (path,file)
             suffix = (start + length new, drop (start + length old) content_)
       selectChunk _ _ = impossible
 
-aPrimPair :: forall wX wY . V1Model wX -> Gen (WithEndState V1Model ((Prim :> Prim) wX) wY)
+aPrimPair :: forall prim wX wY . (PrimPatch prim, ArbitraryState V1Model prim, ApplyState prim ~ RepoState V1Model) => V1Model wX -> Gen (WithEndState V1Model ((prim :> prim) wX) wY)
 aPrimPair repo
   = do mbFile <- maybeOf repoFiles
        frequency
@@ -337,66 +358,143 @@ aPrimPair repo
 ourSmallRepo :: Gen (V1Model wX)
 ourSmallRepo = aSmallRepo
 
-instance ArbitraryState V1Model Prim where
+instance ArbitraryState V1Model Prim1 where
+  arbitraryState s = seal <$> aPrim s
+
+instance ArbitraryState V1Model Prim2 where
   arbitraryState s = seal <$> aPrim s
 
 
-instance Arbitrary (Sealed2 Prim) where
-  arbitrary = makeS2Gen ourSmallRepo
-
-instance Arbitrary (Sealed2 (Prim :> Prim)) where
-  arbitrary = do repo <- ourSmallRepo
-                 WithEndState pp _ <- aPrimPair repo
-                 return $ seal2 pp
-
-instance Arbitrary (Sealed ((Prim :> Prim) wA)) where
-  arbitrary = do repo <- ourSmallRepo
-                 WithEndState pp _ <- aPrimPair repo
-                 return $ seal pp
-
-instance Arbitrary (Sealed2 (Prim :> Prim :> Prim)) where
-  arbitrary = makeS2Gen ourSmallRepo
-
-instance Arbitrary (Sealed ((Prim :> Prim :> Prim) a)) where
+instance Arbitrary (Sealed (Prim1 wA)) where
   arbitrary = makeSGen ourSmallRepo
 
-instance Arbitrary (Sealed2 (FL Prim)) where
-  arbitrary = makeS2Gen ourSmallRepo
-
-instance Arbitrary (Sealed ((FL Prim) wA)) where
+instance Arbitrary (Sealed (Prim2 wA)) where
   arbitrary = makeSGen ourSmallRepo
 
-instance Arbitrary (Sealed2 (FL Prim :> FL Prim)) where
+instance Arbitrary (Sealed2 Prim1) where
   arbitrary = makeS2Gen ourSmallRepo
 
-instance Arbitrary (Sealed ((FL Prim :> FL Prim) wA)) where
+instance Arbitrary (Sealed2 Prim2) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+arbitrarySeal2 :: (PrimPatch prim, ApplyState prim ~ UT.Tree,
+                   ArbitraryState V1Model prim)
+               => Gen (Sealed2 (prim :> prim))
+arbitrarySeal2 = do
+  repo <- ourSmallRepo
+  WithEndState pp _ <- aPrimPair repo
+  return $ seal2 pp
+
+arbitrarySeal :: (PrimPatch prim, ApplyState prim ~ UT.Tree,
+                  ArbitraryState V1Model prim)
+              => Gen (Sealed ((:>) prim prim wX))
+arbitrarySeal = do
+  repo <- ourSmallRepo
+  WithEndState pp _ <- aPrimPair repo
+  return $ seal pp
+
+instance Arbitrary (Sealed2 (Prim1 :> Prim1)) where
+  arbitrary = arbitrarySeal2
+
+instance Arbitrary (Sealed2 (Prim2 :> Prim2)) where
+  arbitrary = arbitrarySeal2
+
+instance Arbitrary (Sealed ((Prim1 :> Prim1) wA)) where
+  arbitrary = arbitrarySeal
+
+instance Arbitrary (Sealed ((Prim2 :> Prim2) wA)) where
+  arbitrary = arbitrarySeal
+
+instance Arbitrary (Sealed2 (Prim1 :> Prim1 :> Prim1)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((Prim1 :> Prim1 :> Prim1) a)) where
   arbitrary = makeSGen ourSmallRepo
 
-instance Arbitrary (Sealed2 (WithState V1Model Prim)) where
+instance Arbitrary (Sealed2 (FL Prim1)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((FL Prim1) wA)) where
+  arbitrary = makeSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (FL Prim1 :> FL Prim1)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((FL Prim1 :> FL Prim1) wA)) where
+  arbitrary = makeSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (WithState V1Model Prim1)) where
   arbitrary = makeWS2Gen ourSmallRepo
 
-instance Arbitrary (Sealed (WithState V1Model Prim wA)) where
+instance Arbitrary (Sealed (WithState V1Model Prim1 wA)) where
   arbitrary = makeWSGen ourSmallRepo
 
-instance Arbitrary (Sealed (WithState V1Model (FL Prim) wA)) where
+instance Arbitrary (Sealed (WithState V1Model (FL Prim1) wA)) where
   arbitrary = makeWSGen ourSmallRepo
 
-instance Arbitrary (Sealed2 (WithState V1Model (Prim :> Prim))) where
+instance Arbitrary (Sealed2 (WithState V1Model (Prim1 :> Prim1))) where
   arbitrary = do repo <- ourSmallRepo
                  WithEndState pp repo' <- aPrimPair repo
                  return $ seal2 $ WithState repo pp repo'
 
-instance Arbitrary (Sealed (WithState V1Model (Prim :> Prim) a)) where
+instance Arbitrary (Sealed (WithState V1Model (Prim1 :> Prim1) a)) where
   arbitrary = do repo <- ourSmallRepo
                  WithEndState pp repo' <- aPrimPair repo
                  return $ seal $ WithState repo pp repo'
 
 
-instance Arbitrary (Sealed2 (WithState V1Model (FL Prim))) where
+instance Arbitrary (Sealed2 (WithState V1Model (FL Prim1))) where
   arbitrary = makeWS2Gen ourSmallRepo
 
-instance Arbitrary (Sealed2 (WithState V1Model (FL Prim :> FL Prim))) where
+instance Arbitrary (Sealed2 (WithState V1Model (FL Prim1 :> FL Prim1))) where
   arbitrary = makeWS2Gen ourSmallRepo
 
-instance Arbitrary (Sealed (WithState V1Model (FL Prim :> FL Prim) a)) where
+instance Arbitrary (Sealed (WithState V1Model (FL Prim1 :> FL Prim1) a)) where
+  arbitrary = makeWSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (Prim2 :> Prim2 :> Prim2)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((Prim2 :> Prim2 :> Prim2) a)) where
+  arbitrary = makeSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (FL Prim2)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((FL Prim2) wA)) where
+  arbitrary = makeSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (FL Prim2 :> FL Prim2)) where
+  arbitrary = makeS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed ((FL Prim2 :> FL Prim2) wA)) where
+  arbitrary = makeSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (WithState V1Model Prim2)) where
+  arbitrary = makeWS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed (WithState V1Model Prim2 wA)) where
+  arbitrary = makeWSGen ourSmallRepo
+
+instance Arbitrary (Sealed (WithState V1Model (FL Prim2) wA)) where
+  arbitrary = makeWSGen ourSmallRepo
+
+instance Arbitrary (Sealed2 (WithState V1Model (Prim2 :> Prim2))) where
+  arbitrary = do repo <- ourSmallRepo
+                 WithEndState pp repo' <- aPrimPair repo
+                 return $ seal2 $ WithState repo pp repo'
+
+instance Arbitrary (Sealed (WithState V1Model (Prim2 :> Prim2) a)) where
+  arbitrary = do repo <- ourSmallRepo
+                 WithEndState pp repo' <- aPrimPair repo
+                 return $ seal $ WithState repo pp repo'
+
+
+instance Arbitrary (Sealed2 (WithState V1Model (FL Prim2))) where
+  arbitrary = makeWS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed2 (WithState V1Model (FL Prim2 :> FL Prim2))) where
+  arbitrary = makeWS2Gen ourSmallRepo
+
+instance Arbitrary (Sealed (WithState V1Model (FL Prim2 :> FL Prim2) a)) where
   arbitrary = makeWSGen ourSmallRepo
