@@ -15,9 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -fno-warn-orphans -fno-warn-deprecations #-}
-{-# LANGUAGE CPP #-}
-
 module Darcs.Test.Patch.Examples.Set1
        ( knownCommutes, knownCantCommutes, knownMerges
        , knownMergeEquivs, knownCanons, mergePairs2
@@ -25,6 +22,8 @@ module Darcs.Test.Patch.Examples.Set1
        , primitiveTestPatches, testPatches, testPatchesNamed
        , primitiveCommutePairs ) where
 
+import Prelude ()
+import Darcs.Prelude
 import System.IO.Unsafe ( unsafePerformIO )
 import qualified Data.ByteString.Char8 as BC ( pack )
 import qualified Data.ByteString as B ( empty )
@@ -34,16 +33,14 @@ import Darcs.Patch
      , readPatch, fromPrim
      , adddir, addfile, hunk, binary, rmdir, rmfile, tokreplace )
 import Darcs.Patch.Prim ( PrimOf, FromPrim )
-import Darcs.Patch.Prim.V1 ( Prim )
 import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import Darcs.Test.Patch.Properties.Check( checkAPatch )
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( unsafeUnseal )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
 
-#include "impossible.h"
-
-type Patch = V1.RepoPatchV1 Prim
+type Patch = V1.RepoPatchV1 V1.Prim
 
 -- The unit tester function is really just a glorified map for functions that
 -- return lists, in which the lists get concatenated (where map would end up
@@ -86,7 +83,7 @@ knownCanons =
      (quickhunk 1 "abcde" "cde" :>: NilFL, quickhunk 1 "ab" "" :>: NilFL),
      (quickhunk 1 "abcde" "acde" :>: NilFL, quickhunk 2 "b" "" :>: NilFL)]
 
-quickhunk :: (FromPrim p, PrimOf p ~ Prim) => Int -> String -> String -> p wX wY
+quickhunk :: (FromPrim p, PrimOf p ~ V1.Prim) => Int -> String -> String -> p wX wY
 quickhunk l o n = fromPrim $ hunk "test" l (map (\c -> BC.pack [c]) o)
                                              (map (\c -> BC.pack [c]) n)
 
