@@ -1,5 +1,6 @@
 module Darcs.Test.Patch.RepoModel where
 import Darcs.Patch.Apply ( Apply, ApplyState )
+import Darcs.Patch.Witnesses.Ordered ( FL )
 import Test.QuickCheck ( Gen )
 
 type Fail = Either String
@@ -21,3 +22,4 @@ class RepoModel model where
 
 type family ModelOf (patch :: * -> * -> *) :: * -> *
 
+type instance ModelOf (FL prim) = ModelOf prim
