@@ -3,6 +3,9 @@ module Darcs.Patch.Inspect
        )
        where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Witnesses.Ordered ( FL, RL, reverseRL, mapFL )
 
 import qualified Data.ByteString.Char8 as BC
