@@ -15,9 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
-
 module Darcs.Patch.Named
        ( Named(..),
          infopatch,
@@ -42,25 +39,30 @@ import Darcs.Patch.Effect ( Effect(effect, effectRL) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Info ( PatchInfo, readPatchInfo, showPatchInfo, patchinfo,
-                          showPatchInfoUI, makePatchname, invertName )
+                          piName, displayPatchInfo, makePatchname, invertName )
 import Darcs.Patch.Merge ( Merge(..) )
-import Darcs.Patch.Patchy ( Commute(..), Invert(..), Apply(..),
-                            PatchInspect(..), ReadPatch(..) )
-import Darcs.Patch.Prim ( PrimOf, PrimPatchBase )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
+import Darcs.Patch.Read ( ReadPatch(..) )
+import Darcs.Patch.Prim ( PrimPatchBase(..) )
 import Darcs.Patch.ReadMonads ( ParserM, option, lexChar,
                                 choice, skipWhile, anyChar )
 import Darcs.Patch.Repair ( mapMaybeSnd, Repair(..), RepairToFL, Check(..) )
-import Darcs.Patch.Show ( ShowPatchBasic(..), ShowPatch(..), showNamedPrefix )
+import Darcs.Patch.Show
+    ( ShowPatchBasic(..), ShowPatch(..), ShowContextPatch(..), ShowPatchFor(..) )
 import Darcs.Patch.Summary ( plainSummary )
 import Darcs.Patch.Viewing () -- for ShowPatch FL instances
 
-import Darcs.Patch.Witnesses.Eq ( MyEq(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( (:>)(..), (:\/:)(..), (:/\:)(..), FL, mapFL, mapFL_FL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed, mapSeal )
 import Darcs.Patch.Witnesses.Show ( ShowDict(..), Show1(..), Show2(..) )
 
-import Darcs.Util.Printer ( ($$), (<+>), (<>), prefix, text, vcat )
+import Darcs.Util.Printer
+    ( Doc, ($$), (<+>), (<>), prefix, text, vcat, cyanText, blueText )
 
 -- | The @Named@ type adds a patch info about a patch, that is a name.
 data Named p wX wY where
@@ -149,7 +151,7 @@ fmapNamed f (NamedP i deps p) = NamedP i deps (mapFL_FL f p)
 fmapFL_Named :: (FL p wA wB -> FL q wC wD) -> Named p wA wB -> Named q wC wD
 fmapFL_Named f (NamedP i deps p) = NamedP i deps (f p)
 
-instance (Commute p, MyEq p) => MyEq (Named p) where
+instance (Commute p, Eq2 p) => Eq2 (Named p) where
     unsafeCompare (NamedP n1 d1 p1) (NamedP n2 d2 p2) =
         n1 == n2 && d1 == d2 && unsafeCompare p1 p2
 
@@ -195,23 +197,73 @@ instance (CommuteNoConflicts p, Conflict p) => Conflict (Named p) where
 instance Check p => Check (Named p) where
     isInconsistent (NamedP _ _ p) = isInconsistent p
 
-instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (Named p) where
-    showPatch (NamedP n [] p) = showPatchInfo n <> showPatch p
-    showPatch (NamedP n d p) = showNamedPrefix n d <+> showPatch p
+-- ForStorage: note the difference between use of <> when there are
+-- no explicit dependencies vs. <+> when there are
+showNamedPrefix :: ShowPatchFor -> PatchInfo -> [PatchInfo] -> Doc -> Doc
+showNamedPrefix f@ForStorage n [] p =
+    showPatchInfo f n <> p
+showNamedPrefix f@ForStorage n d p =
+    showPatchInfo f n
+    $$ blueText "<"
+    $$ vcat (map (showPatchInfo f) d)
+    $$ blueText ">"
+    <+> p
+showNamedPrefix f@ForDisplay n [] p =
+    showPatchInfo f n
+    $$ p
+showNamedPrefix f@ForDisplay n d p =
+    showPatchInfo f n
+    $$ showDependencies ShowDepsVerbose d
+    $$ p
 
-instance (Apply p, CommuteNoConflicts p, Conflict p, IsHunk p, PatchListFormat p,
+data ShowDepsFormat = ShowDepsVerbose | ShowDepsSummary
+                        deriving (Eq)
+
+showDependencies :: ShowDepsFormat -> [PatchInfo] -> Doc
+showDependencies format deps = vcat (map showDependency deps)
+                    where
+                      showDependency d = mark
+                                         <+> cyanText "patch"
+                                         <+> cyanText (show (makePatchname d))
+                                         $$ asterisk <+> text (piName d)
+                      mark | format == ShowDepsVerbose = blueText "depend"
+                           | otherwise = text "D"
+                      asterisk | format == ShowDepsVerbose = text "*"
+                               | otherwise = text "  *"
+
+instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (Named p) where
+    showPatch f (NamedP n d p) = showNamedPrefix f n d $ showPatch f p
+
+instance (Apply p, IsHunk p, PatchListFormat p,
+          ShowContextPatch p) => ShowContextPatch (Named p) where
+    showContextPatch f (NamedP n d p) =
+        showNamedPrefix f n d <$> showContextPatch f p
+
+instance (CommuteNoConflicts p, Conflict p, PatchListFormat p,
           PrimPatchBase p, ShowPatch p) => ShowPatch (Named p) where
-    showContextPatch (NamedP n [] p) = showContextPatch p >>= return . (showPatchInfo n <>)
-    showContextPatch (NamedP n d p) = showContextPatch p >>= return . (showNamedPrefix n d <+>)
-    description (NamedP n _ _) = showPatchInfoUI n
-    summary p = description p $$ text "" $$
-                prefix "    " (plainSummary p) -- this isn't summary because summary does the
-                                            -- wrong thing with (Named (FL p)) so that it can
-                                            -- get the summary of a sequence of named patches
-                                            -- right.
+    description (NamedP n _ _) = displayPatchInfo n
+    summary p@(NamedP _ ds _) =
+        let
+            indent = prefix "    "
+            deps | ds == []  = text ""
+                 | otherwise = text ""
+                               $$ indent (showDependencies ShowDepsSummary ds)
+        in
+            description p $$ deps $$ indent (plainSummary p)
+                                        -- this isn't summary because summary
+                                        -- does the wrong thing with
+                                        -- (Named (FL p)) so that it can get
+                                        -- the summary of a sequence of named
+                                        -- patches right.
     summaryFL = vcat . mapFL summary
-    showNicely p@(NamedP _ _ pt) = description p $$
-                                   prefix "    " (showNicely pt)
+    showNicely p@(NamedP _ ds pt) =
+        let
+            indent = prefix "    "
+            deps | ds == []  = text ""
+                 | otherwise = text ""
+                               $$ indent (showDependencies ShowDepsVerbose ds)
+        in
+            description p <> deps $$ indent (showNicely pt)
 
 instance Show2 p => Show1 (Named p wX) where
     showDict1 = ShowDictClass
