@@ -17,8 +17,6 @@
 -- Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP #-}
-
 
 module Darcs.Patch.Permutations ( removeFL, removeRL, removeCommon,
                                   commuteWhatWeCanFL, commuteWhatWeCanRL,
@@ -38,12 +36,11 @@ import Data.Maybe ( mapMaybe )
 import Darcs.Patch.Commute ( Commute, commute, commuteFLorComplain, commuteRL )
 import Darcs.Patch.CommuteFn ( CommuteFn )
 import Darcs.Patch.Invert ( Invert(..) )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..), (:>)(..), (+<+)
     , reverseFL, (+>+), (:\/:)(..), lengthFL
     , lengthRL, reverseRL )
-#include "impossible.h"
 
 -- |split an 'FL' into "left" and "right" lists according to a predicate @p@, using commutation as necessary.
 -- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
@@ -134,11 +131,11 @@ genCommuteWhatWeCanRL com (xs :<: x :> p) =
 genCommuteWhatWeCanRL _ (NilRL :> y) = NilRL :> y :> NilRL
 
 
-removeCommon :: (MyEq p, Commute p) => (FL p :\/: FL p) wX wY -> (FL p :\/: FL p) wX wY
+removeCommon :: (Eq2 p, Commute p) => (FL p :\/: FL p) wX wY -> (FL p :\/: FL p) wX wY
 removeCommon (xs :\/: NilFL) = xs :\/: NilFL
 removeCommon (NilFL :\/: xs) = NilFL :\/: xs
 removeCommon (xs :\/: ys) = rc xs (headPermutationsFL ys)
-    where rc :: (MyEq p, Commute p) => FL p wX wY -> [(p:>FL p) wX wZ] -> (FL p :\/: FL p) wY wZ
+    where rc :: (Eq2 p, Commute p) => FL p wX wY -> [(p:>FL p) wX wZ] -> (FL p :\/: FL p) wY wZ
           rc nms ((n:>ns):_) | Just ms <- removeFL n nms = removeCommon (ms :\/: ns)
           rc ms [n:>ns] = ms :\/: n:>:ns
           rc ms (_:nss) = rc ms nss
@@ -146,17 +143,17 @@ removeCommon (xs :\/: ys) = rc xs (headPermutationsFL ys)
 
 -- | 'removeFL' @x xs@ removes @x@ from @xs@ if @x@ can be commuted to its head.
 --   Otherwise it returns 'Nothing'
-removeFL :: (MyEq p, Commute p) => p wX wY -> FL p wX wZ -> Maybe (FL p wY wZ)
+removeFL :: (Eq2 p, Commute p) => p wX wY -> FL p wX wZ -> Maybe (FL p wY wZ)
 removeFL x xs = r x $ headPermutationsFL xs
-    where r :: (MyEq p, Commute p) => p wX wY -> [(p:>FL p) wX wZ] -> Maybe (FL p wY wZ)
+    where r :: (Eq2 p, Commute p) => p wX wY -> [(p:>FL p) wX wZ] -> Maybe (FL p wY wZ)
           r _ [] = Nothing
           r z ((z':>zs):zss) | IsEq <- z =\/= z' = Just zs
                              | otherwise = r z zss
 
 -- | 'removeRL' is like 'removeFL' except with 'RL'
-removeRL :: (MyEq p, Commute p) => p wY wZ -> RL p wX wZ -> Maybe (RL p wX wY)
+removeRL :: (Eq2 p, Commute p) => p wY wZ -> RL p wX wZ -> Maybe (RL p wX wY)
 removeRL x xs = r x $ headPermutationsRL xs
-    where r :: (MyEq p, Commute p) => p wY wZ -> [RL p wX wZ] -> Maybe (RL p wX wY)
+    where r :: (Eq2 p, Commute p) => p wY wZ -> [RL p wX wZ] -> Maybe (RL p wX wY)
           r z ((zs:<:z'):zss) | IsEq <- z =/\= z' = Just zs
                               | otherwise = r z zss
           r _ _ = Nothing
@@ -165,21 +162,21 @@ removeRL x xs = r x $ headPermutationsRL xs
 --   @ab@ have been commuted out of it, if possible.  If this is not possible
 --   for any reason (the set of patches @ab@ is not actually a subset of @abc@,
 --   or they can't be commuted out) we return 'Nothing'.
-removeSubsequenceFL :: (MyEq p, Commute p) => FL p wA wB
+removeSubsequenceFL :: (Eq2 p, Commute p) => FL p wA wB
                      -> FL p wA wC -> Maybe (FL p wB wC)
 removeSubsequenceFL a b | lengthFL a > lengthFL b = Nothing
                          | otherwise = rsFL a b
-    where rsFL :: (MyEq p, Commute p) => FL p wA wB -> FL p wA wC -> Maybe (FL p wB wC)
+    where rsFL :: (Eq2 p, Commute p) => FL p wA wB -> FL p wA wC -> Maybe (FL p wB wC)
           rsFL NilFL ys = Just ys
           rsFL (x:>:xs) yys = removeFL x yys >>= removeSubsequenceFL xs
 
 -- | 'removeSubsequenceRL' is like @removeSubsequenceFL@ except that it works
 --   on 'RL'
-removeSubsequenceRL :: (MyEq p, Commute p) => RL p wAb wAbc
+removeSubsequenceRL :: (Eq2 p, Commute p) => RL p wAb wAbc
                      -> RL p wA wAbc -> Maybe (RL p wA wAb)
 removeSubsequenceRL a b | lengthRL a > lengthRL b = Nothing
                          | otherwise = rsRL a b
-    where rsRL :: (MyEq p, Commute p) => RL p wAb wAbc -> RL p wA wAbc -> Maybe (RL p wA wAb)
+    where rsRL :: (Eq2 p, Commute p) => RL p wAb wAbc -> RL p wA wAbc -> Maybe (RL p wA wAb)
           rsRL NilRL ys = Just ys
           rsRL (xs:<:x) yys = removeRL x yys >>= removeSubsequenceRL xs
 
@@ -222,7 +219,7 @@ headPermutationsRL (ps:<:p) =
                                               Just $ xs:<:p1':<:p2'
               swapfirstRL _ = Nothing
 
-instance (MyEq p, Commute p) => MyEq (FL p) where
+instance (Eq2 p, Commute p) => Eq2 (FL p) where
     a =\/= b | lengthFL a /= lengthFL b = NotEq
              | otherwise = cmpSameLength a b
              where cmpSameLength :: FL p wX wY -> FL p wX wZ -> EqCheck wY wZ
@@ -231,7 +228,7 @@ instance (MyEq p, Commute p) => MyEq (FL p) where
                    cmpSameLength _ _ = NotEq
     xs =/\= ys = reverseFL xs =/\= reverseFL ys
 
-instance (MyEq p, Commute p) => MyEq (RL p) where
+instance (Eq2 p, Commute p) => Eq2 (RL p) where
     unsafeCompare = bug "Buggy use of unsafeCompare on RL"
     a =/\= b | lengthRL a /= lengthRL b = NotEq
              | otherwise = cmpSameLength a b
