@@ -1,7 +1,7 @@
 --  Copyright (C) 2009 Ganesh Sittampalam
 --
 --  BSD3
-{-# LANGUAGE CPP, GADTs, UndecidableInstances #-}
+{-# LANGUAGE UndecidableInstances #-}
 module Darcs.Patch.Rebase.Viewing
     ( RebaseSelect(..)
     , toRebaseSelect, fromRebaseSelect, extractRebaseSelect, reifyRebaseSelect
@@ -44,12 +44,14 @@ import qualified Darcs.Patch.Named.Wrapped as Wrapped
     )
 
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
-import Darcs.Patch.Patchy ( Invert(..), Commute(..), Patchy, Apply(..),
-                            ShowPatch(..), ReadPatch(..),
-                            PatchInspect(..)
-                          )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
+import Darcs.Patch.Read ( ReadPatch(..) )
+import Darcs.Patch.Show ( ShowPatch(..) )
 import Darcs.Patch.Prim
-    ( PrimPatch, PrimPatchBase, PrimOf, FromPrim(..), FromPrims(..)
+    ( PrimPatch, PrimPatchBase(..), FromPrim(..), FromPrims(..)
     )
 import Darcs.Patch.Rebase.Container ( Suspended(..) )
 import Darcs.Patch.Rebase.Fixup
@@ -60,7 +62,7 @@ import Darcs.Patch.Rebase.Fixup
 import Darcs.Patch.Rebase.Item ( RebaseItem(..) )
 import Darcs.Patch.Rebase.Name ( RebaseName(..) )
 import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
-import Darcs.Patch.Show ( ShowPatchBasic(..) )
+import Darcs.Patch.Show ( ShowPatchBasic(..), ShowPatchFor(..), ShowContextPatch(..) )
 import Darcs.Patch.Summary ( plainSummary )
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Ordered
@@ -77,8 +79,6 @@ import Darcs.Util.Show ( appPrec )
 import Data.List ( nub, (\\) )
 import Data.Maybe ( fromMaybe )
 
-#include "impossible.h"
-
 -- |Encapsulate a single patch in the rebase state together with its fixups.
 -- Used during interactive selection to make sure that each item presented
 -- to the user corresponds to a patch.
@@ -142,14 +142,6 @@ rsToPia (RSRev _ toEdit) = Sealed2 (n2pia (NormalP toEdit))
 instance PrimPatchBase p => PrimPatchBase (RebaseSelect p) where
    type PrimOf (RebaseSelect p) = PrimOf p
 
-instance (PrimPatchBase p, PatchListFormat p, Conflict p, FromPrim p, Effect p, CommuteNoConflicts p, IsHunk p, Patchy p, ApplyState p ~ ApplyState (PrimOf p))
-    => Patchy (RebaseSelect p)
-
-instance ( PrimPatchBase p, Apply p, ApplyState p ~ ApplyState (PrimOf p)
-         , Invert p
-         )
-        => Patchy (RebaseChange p)
-
 instance PatchDebug p => PatchDebug (RebaseSelect p)
 
 instance PatchDebug p => PatchDebug (RebaseChange p)
@@ -168,7 +160,7 @@ instance ( PrimPatchBase p, Invert p, Apply p
     apply (RCFwd fixups contents) = apply fixups >> apply contents
     apply (RCRev fixups contents) = apply (invert contents) >> apply (invertFL fixups)
 
-instance (PrimPatchBase p, FromPrim p, Conflict p, CommuteNoConflicts p, Invert p) => Conflict (RebaseSelect p) where
+instance (PrimPatchBase p, Conflict p, CommuteNoConflicts p, Invert p) => Conflict (RebaseSelect p) where
    resolveConflicts (RSFwd _ toedit) = resolveConflicts toedit
    resolveConflicts (RSRev{}) = impossible
 
@@ -227,14 +219,19 @@ instance (PrimPatchBase p, Invert p, Effect p) => Effect (RebaseChange p) where
         invertRL . reverseFL . effect $ RCFwd fixups changes
 
 instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (RebaseSelect p) where
-   showPatch (RSFwd fixups toedit) =
-         showPatch (Items (mapFL_FL Fixup fixups +>+ ToEdit toedit :>: NilFL))
-   showPatch (RSRev {}) = impossible
+   showPatch f (RSFwd fixups toedit) =
+         showPatch f (Items (mapFL_FL Fixup fixups +>+ ToEdit toedit :>: NilFL))
+   showPatch _ (RSRev {}) = impossible
 
-instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p)
+-- TODO this is a dummy instance that does not actually show context
+instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p) => ShowContextPatch (RebaseSelect p) where
+   showContextPatch f p = return $ showPatch f p
+
+instance (PrimPatchBase p, ShowPatchBasic p)
       => ShowPatchBasic (RebaseChange p) where
-    showPatch (RCFwd fixups contents) =
-        vcat (mapFL showPatch contents) $$
+    showPatch ForStorage _ = impossible
+    showPatch ForDisplay (RCFwd fixups contents) =
+        vcat (mapFL (showPatch ForDisplay) contents) $$
         (if nullFL fixups
             then empty
             else
@@ -244,9 +241,9 @@ instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p)
                 vcat (mapRL showFixup (invertFL fixups))
         )
         where
-            showFixup (PrimFixup p) = showPatch p
-            showFixup (NameFixup n) = showPatch n
-    showPatch (RCRev {}) = impossible
+            showFixup (PrimFixup p) = showPatch ForDisplay p
+            showFixup (NameFixup n) = showPatch ForDisplay n
+    showPatch _ (RCRev {}) = impossible
 
 instance (PrimPatchBase p, PatchListFormat p, Apply p, CommuteNoConflicts p, Conflict p, IsHunk p, ShowPatch p)
     => ShowPatch (RebaseSelect p) where
@@ -267,8 +264,14 @@ instance
     summary = plainSummary
     summaryFL = plainSummary
 
+-- TODO this is a dummy instance that does not actually show context
+instance
+    ( PrimPatchBase p, ShowPatchBasic p)
+   => ShowContextPatch (RebaseChange p) where
 
-instance ReadPatch p => ReadPatch (RebaseSelect p) where
+    showContextPatch f p = return $ showPatch f p
+
+instance ReadPatch (RebaseSelect p) where
    readPatch' = error "can't read RebaseSelect patches"
 
 instance ReadPatch (RebaseChange p) where
@@ -320,7 +323,7 @@ instance PrimPatch (PrimOf p) => PrimPatchBase (RebaseChange p) where
     type PrimOf (RebaseChange p) = PrimOf p
 
 
-instance Invert p => Invert (RebaseSelect p) where
+instance Invert (RebaseSelect p) where
    invert (RSFwd fixups edit) = RSRev fixups edit
    invert (RSRev fixups edit) = RSFwd fixups edit
 
@@ -328,7 +331,7 @@ instance Invert (RebaseChange p) where
    invert (RCFwd fixups contents) = RCRev fixups contents
    invert (RCRev fixups contents) = RCFwd fixups contents
 
-instance (PrimPatchBase p, Commute p, MyEq p) => MyEq (RebaseSelect p) where
+instance (PrimPatchBase p, Commute p, Eq2 p) => Eq2 (RebaseSelect p) where
    RSFwd fixups1 edit1 =\/= RSFwd fixups2 edit2
       | IsEq <- fixups1 =\/= fixups2, IsEq <- edit1 =\/= edit2 = IsEq
    RSRev fixups1 edit1 =\/= RSRev fixups2 edit2
@@ -460,7 +463,7 @@ forceCommuteName (Rename old new :> WithDroppedDeps (NamedP pn deps body) ddeps)
       let newdeps = map (\dep -> if new == dep then old else dep) deps
       in WithDroppedDeps (NamedP pn newdeps (unsafeCoerceP body)) ddeps :> Rename old new
 
-forceCommutePrim :: (Commute p, Merge p, Invert p, Effect p, FromPrim p)
+forceCommutePrim :: (Merge p, Invert p, Effect p, FromPrim p)
                  => (PrimOf p :> WDDNamed p) wX wY
                  -> (WDDNamed p :> FL (PrimOf p)) wX wY
 forceCommutePrim (p :> q) =
@@ -468,7 +471,7 @@ forceCommutePrim (p :> q) =
         q' :/\: invp' -> q' :> effect (invert invp')
 
 
-forceCommutesPrim :: (Commute p, Merge p, Invert p, Effect p, FromPrim p)
+forceCommutesPrim :: (Merge p, Invert p, Effect p, FromPrim p)
                   => (PrimOf p :> FL (WDDNamed p)) wX wY
                   -> (FL (WDDNamed p) :> FL (PrimOf p)) wX wY
 forceCommutesPrim (p :> NilFL) = NilFL :> (p :>: NilFL)
@@ -477,7 +480,7 @@ forceCommutesPrim (p :> (q :>: qs)) =
         q' :> p' -> case forceCommutessPrim ( p' :> qs) of
             qs' :> p'' -> (q' :>: qs') :> p''
 
-forceCommutessPrim :: (Commute p, Merge p, Invert p, Effect p, FromPrim p)
+forceCommutessPrim :: (Merge p, Invert p, Effect p, FromPrim p)
                    => (FL (PrimOf p) :> FL (WDDNamed p)) wX wY
                    -> (FL (WDDNamed p) :> FL (PrimOf p)) wX wY
 forceCommutessPrim (NilFL :> qs) = qs :> NilFL
@@ -487,7 +490,7 @@ forceCommutessPrim ((p :>: ps) :> qs) =
             case forceCommutesPrim (p :> qs') of
                 qs'' :> p' -> qs'' :> (p' +>+ ps')
 
-forceCommutess :: (Commute p, Merge p, Invert p, Effect p, FromPrim p)
+forceCommutess :: (Merge p, Invert p, Effect p, FromPrim p)
                => (FL (RebaseFixup p) :> FL (WDDNamed p)) wX wY
                -> (FL (WDDNamed p) :> FL (RebaseFixup p)) wX wY
 forceCommutess (NilFL :> qs) = qs :> NilFL
@@ -506,7 +509,7 @@ forceCommutess ((PrimFixup p :>: ps) :> qs) =
 -- the main repository, together with residual fixups that need
 -- to go back into the rebase state (unless the rebase is now finished).
 -- Any fixups associated with the patch will turn into conflicts.
-extractRebaseSelect :: (Commute p, Merge p, Invert p, Effect p, FromPrim p, PrimPatchBase p)
+extractRebaseSelect :: (Merge p, Invert p, Effect p, FromPrim p, PrimPatchBase p)
                     => FL (RebaseSelect p) wX wY
                     -> (FL (WDDNamed p) :> FL (RebaseFixup p)) wX wY
 extractRebaseSelect NilFL = NilFL :> NilFL
@@ -522,7 +525,7 @@ extractRebaseSelect (RSRev{} :>: _) = impossible
 -- signature to be compatible with extractRebaseSelect
 -- | Like 'extractRebaseSelect', but any fixups are "reified" into a separate patch.
 reifyRebaseSelect :: forall p wX wY
-                   . (PrimPatchBase p, Commute p, Merge p, Invert p, Effect p, FromPrim p)
+                   . (PrimPatchBase p, FromPrim p)
                   => FL (RebaseSelect p) wX wY
                   -> IO ((FL (WDDNamed p) :> FL (RebaseFixup p)) wX wY)
 reifyRebaseSelect rs = do res <- concatFL <$> mapFL_FL_M reifyOne rs
@@ -553,7 +556,7 @@ mkDummy (Rename _ _) = error "internal error: can't make a dummy patch from a re
 instance CommuteNoConflicts (RebaseChange p) where
     commuteNoConflicts _ = impossible
 
-instance IsHunk p => IsHunk (RebaseChange p) where
+instance IsHunk (RebaseChange p) where
     -- RebaseChange is a compound patch, so it doesn't really make sense to
     -- ask whether it's a hunk. TODO: get rid of the need for this.
     isHunk _ = Nothing
