@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, RankNTypes, GADTs, ViewPatterns #-}
+{-# LANGUAGE ViewPatterns #-}
 {-# OPTIONS_GHC -fno-warn-unused-imports #-}
 -- Copyright (C) 2009 Ganesh Sittampalam
 --
@@ -22,7 +22,13 @@
 -- CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 -- SOFTWARE.
 
-module Darcs.Patch.Split ( Splitter(..), rawSplitter, noSplitter, primSplitter, reversePrimSplitter ) where
+module Darcs.Patch.Split
+    ( Splitter(..)
+    , rawSplitter
+    , noSplitter
+    , primSplitter
+    , reversePrimSplitter
+    ) where
 
 import Prelude ()
 import Darcs.Prelude
@@ -33,14 +39,16 @@ import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed
 
 import Darcs.Patch.FileHunk ( FileHunk(..), IsHunk(..) )
-import Darcs.Patch.Patchy ( ReadPatch(..), showPatch, ShowPatch(..), Invert(..) )
-import Darcs.Patch.Invert (invertFL)
+import Darcs.Patch.Read ( ReadPatch(..) )
+import Darcs.Patch.Show ( showPatch, ShowPatch(..) )
+import Darcs.Patch.Invert( Invert(..), invertFL )
 import Darcs.Patch.Prim ( PrimPatch, canonize, canonizeFL, primFromHunk )
 import Darcs.Patch.ReadMonads ( parseStrictly )
 import Darcs.Patch.Read ()
+import Darcs.Patch.Show ( ShowPatchFor(ForDisplay) )
 import Darcs.Patch.Viewing ()
 
-import Darcs.Util.Printer ( renderPS, RenderMode(..) )
+import Darcs.Util.Printer ( renderPS )
 import qualified Darcs.Util.Diff as D ( DiffAlgorithm )
 
 import qualified Data.ByteString as B
@@ -63,23 +71,20 @@ import qualified Data.ByteString.Char8 as BC
 -- Prim (you shouldn't go editing named patches, you'll break them!)
 -- However you might want to compose splitters for FilePatchType to make
 -- splitters for Prim etc, and the generality doesn't cost anything.
-data Splitter p
-  = Splitter {
-              applySplitter :: forall wX wY . p wX wY
-                              -> Maybe (B.ByteString,
-                                        B.ByteString -> Maybe (FL p wX wY))
-              -- canonization is needed to undo the effects of splitting
-              -- Typically, the list returned by applySplitter will not
-              -- be in the simplest possible form (since the user will have
-              -- deliberately added extra stuff). Once the user has selected
-              -- the pieces they want, we need to make sure that we eliminate
-              -- any remaining redundancy in the selected pieces, otherwise
-              -- we might record (or whatever) a rather strange looking patch.
-              -- This hook allows the splitter to provide an appropriate
-              -- function for doing this.
-             ,canonizeSplit :: forall wX wY . FL p wX wY
-                              -> FL p wX wY
-             }
+data Splitter p = Splitter
+    { applySplitter :: forall wX wY. p wX wY
+                    -> Maybe (B.ByteString, B.ByteString -> Maybe (FL p wX wY))
+      -- canonization is needed to undo the effects of splitting
+      -- Typically, the list returned by applySplitter will not
+      -- be in the simplest possible form (since the user will have
+      -- deliberately added extra stuff). Once the user has selected
+      -- the pieces they want, we need to make sure that we eliminate
+      -- any remaining redundancy in the selected pieces, otherwise
+      -- we might record (or whatever) a rather strange looking patch.
+      -- This hook allows the splitter to provide an appropriate
+      -- function for doing this.
+    , canonizeSplit :: forall wX wY. FL p wX wY -> FL p wX wY
+    }
 
 {- Some facts that probably ought to be true about splitters: should make some QC
 properties
@@ -94,22 +99,21 @@ applySplitter p = Just (bs, f) ; f bs' = Just ps ==> canonizeSplit ps = p :>: Ni
 withEditedHead :: Invert p => p wX wY -> p wX wZ -> FL p wX wY
 withEditedHead p res = res :>: invert res :>: p :>: NilFL
 
--- |This generic splitter just lets the user edit the printed representation of the patch
--- Should not be used expect for testing and experimentation.
+-- |This generic splitter just lets the user edit the printed representation of the
+-- patch. Should not be used expect for testing and experimentation.
 rawSplitter :: (ShowPatch p, ReadPatch p, Invert p) => Splitter p
-rawSplitter = Splitter {
-                  applySplitter =
-                     \p -> Just (renderPS Standard . showPatch $ p,
-                                 \str -> case parseStrictly readPatch' str of
-                                          Just (Sealed res, _) -> Just (withEditedHead p res)
-                                          _ -> Nothing
-                                )
-                 ,canonizeSplit = id
-                }
+rawSplitter = Splitter
+    { applySplitter = \p ->
+        Just (renderPS . showPatch ForDisplay $ p
+             ,\str -> case parseStrictly readPatch' str of
+                        Just (Sealed res, _) -> Just (withEditedHead p res)
+                        _                    -> Nothing)
+    , canonizeSplit = id
+    }
 
--- |Never splits. In other code we normally pass around Maybe Splitter instead of using this
--- as the default, because it saves clients that don't care about splitting from having to
--- import this module just to get noSplitter.
+-- |Never splits. In other code we normally pass around Maybe Splitter instead
+-- of using this as the default, because it saves clients that don't care about
+-- splitting from having to import this module just to get noSplitter.
 noSplitter :: Splitter p
 noSplitter = Splitter { applySplitter = const Nothing, canonizeSplit = id }
 
@@ -118,16 +122,17 @@ doPrimSplit :: PrimPatch prim => D.DiffAlgorithm -> prim wX wY
             -> Maybe (B.ByteString, B.ByteString -> Maybe (FL prim wX wY))
 doPrimSplit da = doPrimSplit_ da True explanation
   where
-    explanation = map BC.pack
-                   [ "Interactive hunk edit:"
-                   , " - Edit the section marked 'AFTER'"
-                   , " - Arbitrary editing is supported"
-                   , " - This will only affect the patch, not your working tree"
-                   , " - Hints:"
-                   , "   - To split added text, delete the part you want to postpone"
-                   , "   - To split removed text, copy back the part you want to retain"
-                   , ""
-                   ]
+    explanation =
+      map BC.pack
+        [ "Interactive hunk edit:"
+        , " - Edit the section marked 'AFTER'"
+        , " - Arbitrary editing is supported"
+        , " - This will only affect the patch, not your working tree"
+        , " - Hints:"
+        , "   - To split added text, delete the part you want to postpone"
+        , "   - To split removed text, copy back the part you want to retain"
+        , ""
+        ]
 
 doPrimSplit_ :: (PrimPatch prim, IsHunk p)
              => D.DiffAlgorithm
@@ -161,9 +166,8 @@ doPrimSplit_ da edit_before_part helptext (isHunk -> Just (FileHunk fn n before
                            (ys, _:zs) -> Just (ys, zs)
 doPrimSplit_ _ _ _ _ = Nothing
 
--- |Split a primitive hunk patch up
--- by allowing the user to edit both the before and after lines, then insert fixup patches
--- to clean up the mess.
+-- |Split a primitive hunk patch up by allowing the user to edit both the
+-- before and after lines, then insert fixup patches to clean up the mess.
 primSplitter :: PrimPatch p => D.DiffAlgorithm -> Splitter p
 primSplitter da = Splitter { applySplitter = doPrimSplit da
                            , canonizeSplit = canonizeFL da }
@@ -178,16 +182,17 @@ doReversePrimSplit da prim = do
   return (text, parser')
   where
     reverseExplanation =
-      map BC.pack [ "Interactive hunk edit:"
-                  , " - Edit the section marked 'AFTER' (representing the state to which you'll revert)"
-                  , " - Arbitrary editing is supported"
-                  , " - Your working tree will be returned to the 'AFTER' state"
-                  , " - Do not touch the 'BEFORE' section"
-                  , " - Hints:"
-                  , "   - To revert only a part of a text addition, delete the part you want to get rid of"
-                  , "   - To revert only a part of a removal, copy back the part you want to retain"
-                  , ""
-                                 ]
+      map BC.pack
+        [ "Interactive hunk edit:"
+        , " - Edit the section marked 'AFTER' (representing the state to which you'll revert)"
+        , " - Arbitrary editing is supported"
+        , " - Your working tree will be returned to the 'AFTER' state"
+        , " - Do not touch the 'BEFORE' section"
+        , " - Hints:"
+        , "   - To revert only a part of a text addition, delete the part you want to get rid of"
+        , "   - To revert only a part of a removal, copy back the part you want to retain"
+        , ""
+        ]
 
 reversePrimSplitter :: PrimPatch prim => D.DiffAlgorithm -> Splitter prim
 reversePrimSplitter da = Splitter { applySplitter = doReversePrimSplit da
