@@ -16,8 +16,6 @@
 --  Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP #-}
-
 
 module Darcs.Patch.V1.Commute
     (
@@ -36,9 +34,12 @@ import Control.Applicative ( Alternative(..) )
 import Darcs.Patch.Commute ( selfCommuter )
 import Darcs.Patch.CommuteFn ( commuterIdFL, commuterFLId )
 import Darcs.Util.Path ( FileName )
+import Darcs.Util.Printer ( errorDoc )
 import Darcs.Patch.Invert ( invertRL )
-import Darcs.Patch.Merge ( Merge(..) )
-import Darcs.Patch.Patchy ( Commute(..), PatchInspect(..), Invert(..) )
+import Darcs.Patch.Merge ( Merge(..), naturalMerge )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.V1.Core ( RepoPatchV1(..),
                              isMerger,
                              mergerUndo )
@@ -57,11 +58,10 @@ import Darcs.Util.Printer ( text, vcat, ($$) )
 import Darcs.Patch.V1.Show ( showPatch_ )
 import Data.List ( nub, nubBy )
 import Data.List.Ordered ( nubSort )
-#include "impossible.h"
 import Darcs.Patch.Witnesses.Sealed
     ( Sealed(..) , mapSeal, unseal, FlippedSeal(..), mapFlipped
     , unsafeUnseal, unsafeUnsealFlipped )
-import Darcs.Patch.Witnesses.Eq ( EqCheck(..), MyEq(..) )
+import Darcs.Patch.Witnesses.Eq ( EqCheck(..), Eq2(..) )
 import Darcs.Patch.Witnesses.Unsafe
     ( unsafeCoerceP, unsafeCoercePStart
     , unsafeCoercePEnd )
@@ -182,7 +182,7 @@ instance PrimPatch prim => Merge (RepoPatchV1 prim) where
         -- "after" @z@), but we need to return both arms. We therefore commute
         -- @z@ and @y'@, to obtain a @z'@, which applies "after" @y'' == y@.
         Sealed y' -> case commute (z :> y') of
-                         Nothing -> bugDoc $ text "merge_patches bug"
+                         Nothing -> errorDoc $ text "merge_patches bug"
                                     $$ showPatch_ y
                                    $$ showPatch_ z
                                    $$ showPatch_ y'
@@ -272,17 +272,6 @@ commuteFLId commuter (p :> (q :>: qs)) = do
    qs' :> p'' <- commuteFLId commuter (p' :> qs)
    return ((q' :>: qs') :> p'')
 
--- | elegantMerge attempts to perform the "intuitive" merge of two patches,
--- from a common starting context @wX@.
-elegantMerge :: PrimPatch prim
-             => (RepoPatchV1 prim :\/: RepoPatchV1 prim) wX wY
-             -> Maybe ((RepoPatchV1 prim :/\: RepoPatchV1 prim) wX wY)
-elegantMerge (p1 :\/: p2) = do
-  p1' :> ip2' <- commute (invert p2 :> p1)
-  p1o :> _    <- commute (p2 :> p1')
-  guard $ unsafeCompare p1o p1 -- should be a redundant check
-  return $ invert ip2' :/\: p1'
-
 {-
 A note about mergers and type witnesses
 ---------------------------------------
@@ -313,7 +302,7 @@ these constructors.
 -- rebuild it?
 actualMerge :: PrimPatch prim
     => (RepoPatchV1 prim :\/: RepoPatchV1 prim) wX wY -> Sealed (RepoPatchV1 prim wY)
-actualMerge (p1 :\/: p2) = case elegantMerge (p1:\/:p2) of
+actualMerge (p1 :\/: p2) = case naturalMerge (p1:\/:p2) of
                              Just (_ :/\: p1') -> Sealed p1'
                              Nothing -> merger "0.0" p2 p1
 
@@ -352,7 +341,7 @@ reconcileUnwindings p (p1s:<:p1) p2s@(tp2s:<:p2) =
               Just p1s' -> mapFlipped (:<:p2) $
                            reconcileUnwindings p p1s' tp2s
               Nothing ->
-                  bugDoc $ text "in function reconcileUnwindings"
+                errorDoc $ text "in function reconcileUnwindings"
                         $$ text "Original patch:"
                         $$ showPatch_ p
     _ -> bug "in reconcileUnwindings"
@@ -381,7 +370,8 @@ instance PrimPatch prim => Conflict (RepoPatchV1 prim) where
               case commuteFLId commuteNoMerger (p :> passedby) of
               Just (_ :> p'@(Merger _ _ p1 p2)) ->
                   map Sealed (nubBy unsafeCompare $
-                        effect (unsafeCoercePStart $ unsafeUnseal (glump09 p1 p2)) : map (unsafeCoercePStart . unsafeUnseal) (unravel p'))
+                        effect (unsafeCoercePStart $ unsafeUnseal (glump09 p1 p2)) :
+                          map (unsafeCoercePStart . unsafeUnseal) (unravel p'))
                   : rcs (p :>: passedby) ps
               Nothing -> rcs (p :>: passedby) ps
               _ -> impossible
@@ -395,7 +385,7 @@ instance PrimPatch prim => Conflict (RepoPatchV1 prim) where
 
 
 -- This type seems wrong - the most natural type for the result would seem to be
--- [Sealed (FL Prim wX)], given the type of unwind.
+-- [Sealed (FL prim wX)], given the type of unwind.
 -- However downstream code in darcs convert assumes the wY type, and I was unable
 -- to figure out whether this could/should reasonably be changed -- Ganesh 13/4/10
 publicUnravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wY)]
@@ -463,11 +453,11 @@ newUr :: PrimPatch prim
 newUr p (ps :<: Merger _ _ p1 p2) =
    case filter (\(_:<:pp) -> pp `unsafeCompare` p1) $ headPermutationsRL ps of
    ((ps':<:_):_) -> newUr p (ps':<:unsafeCoercePStart p1) ++ newUr p (ps':<:unsafeCoercePStart p2)
-   _ -> bugDoc $ text "in function newUr"
-              $$ text "Original patch:"
-              $$ showPatch_ p
-              $$ text "Unwound:"
-              $$ vcat (unseal (mapRL showPatch_) $ unwind p)
+   _ -> errorDoc $ text "in function newUr"
+                 $$ text "Original patch:"
+                 $$ showPatch_ p
+                 $$ text "Unwound:"
+                 $$ vcat (unseal (mapRL showPatch_) $ unwind p)
 
 newUr op ps =
     case filter (\(_:<:p) -> isMerger p) $ headPermutationsRL ps of
@@ -481,13 +471,13 @@ instance Invert prim => Invert (RepoPatchV1 prim) where
         = Merger undo unwindings p1 p2
     invert (PP p) = PP (invert p)
 
-instance MyEq prim => MyEq (RepoPatchV1 prim) where
+instance Eq2 prim => Eq2 (RepoPatchV1 prim) where
     unsafeCompare = eqPatches
 
-instance MyEq prim => Eq (RepoPatchV1 prim wX wY) where
+instance Eq2 prim => Eq (RepoPatchV1 prim wX wY) where
     (==) = unsafeCompare
 
-eqPatches :: MyEq prim => RepoPatchV1 prim wX wY -> RepoPatchV1 prim wW wZ -> Bool
+eqPatches :: Eq2 prim => RepoPatchV1 prim wX wY -> RepoPatchV1 prim wW wZ -> Bool
 eqPatches (PP p1) (PP p2) = unsafeCompare p1 p2
 eqPatches (Merger _ _ p1a p1b) (Merger _ _ p2a p2b)
  = eqPatches p1a p2a &&
