@@ -15,9 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-
-{-# LANGUAGE CPP #-}
-
 module Darcs.Repository.Prefs
     ( addToPreflist
     , deleteSources
@@ -39,12 +36,15 @@ module Darcs.Repository.Prefs
     , FileType(..)
     , filetypeFunction
     , getCaches
-    , binariesFileHelp
-    , boringFileHelp
     , globalCacheDir
     , globalPrefsDirDoc
     , globalPrefsDir
-    , oldGlobalCacheDir
+    , getMotd
+    , showMotd
+    , prefsUrl
+    , prefsDirPath
+    -- * documentation of prefs files
+    , prefsFilesHelp
     ) where
 
 import Prelude ()
@@ -56,28 +56,28 @@ import Data.Char ( toUpper )
 import Data.List ( nub, isPrefixOf, union, sortBy, lookup )
 import Data.Maybe ( isJust, fromMaybe, mapMaybe, catMaybes, maybeToList )
 import qualified Control.Exception as C
-import qualified Data.ByteString       as B  ( empty )
+import qualified Data.ByteString       as B  ( empty, null, hPut, ByteString )
 import qualified Data.ByteString.Char8 as BC ( unpack )
 import System.Directory ( getAppUserDataDirectory, doesDirectoryExist,
                           createDirectory, doesFileExist )
 import System.Environment ( getEnvironment )
 import System.FilePath.Posix ( normalise, dropTrailingPathSeparator, (</>) )
 import System.IO.Error ( isDoesNotExistError )
-import System.IO ( stderr )
+import System.IO ( stdout, stderr )
 import System.Info ( os )
 import Text.Regex ( Regex, mkRegex, matchRegex )
 
 import Darcs.Repository.Cache ( Cache(..), CacheType(..), CacheLoc(..),
                                 WritableOrNot(..), compareByLocality )
-import Darcs.Util.External ( gzFetchFilePS , Cachable( Cachable ))
+import Darcs.Util.External ( gzFetchFilePS , fetchFilePS, Cachable(..))
 import Darcs.Repository.Flags( UseCache (..), DryRun (..), SetDefault (..),
                                RemoteRepos (..) )
-import Darcs.Util.Lock( readBinFile, writeBinFile )
+import Darcs.Util.Lock( readTextFile, writeTextFile )
 import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.Path ( AbsolutePath, ioAbsolute, toFilePath,
                          getCurrentDirectory )
-import Darcs.Util.Printer( hPutDocLn, text, RenderMode(..) )
+import Darcs.Util.Printer( hPutDocLn, text )
 import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.Util.File ( osxCacheDir, xdgCacheDir, removeFileMayNotExist )
 
@@ -91,9 +91,8 @@ writeDefaultPrefs = do
     setPreflist "binaries" defaultBinaries
     setPreflist "motd" []
 
-{-# NOINLINE defaultBoring #-}
 defaultBoring :: [String]
-defaultBoring = map ("# " ++) boringFileHelp ++
+defaultBoring = map ("# " ++) boringFileInternalHelp ++
     [ ""
     , "### compiler and interpreter intermediate files"
     , "# haskell (ghc) interfaces"
@@ -156,8 +155,6 @@ defaultBoring = map ("# " ++) boringFileHelp ++
     , "(^|/)SCCS($|/)"
     , "# darcs"
     , "(^|/)"++darcsdir++"($|/)", "(^|/)\\.darcsrepo($|/)"
-    , "^\\.darcs-temp-mail$"
-    , "-darcs-backup[[:digit:]]+$"
     , "# gnu arch"
     , "(^|/)(\\+|,)"
     , "(^|/)vssver\\.scc$"
@@ -191,12 +188,12 @@ defaultBoring = map ("# " ++) boringFileHelp ++
     , "(^|.*/)\\.emacs\\.desktop(\\.lock)?$"
     ]
 
-boringFileHelp :: [String]
-boringFileHelp =
+boringFileInternalHelp :: [String]
+boringFileInternalHelp =
     [ "This file contains a list of extended regular expressions, one per"
     , "line. A file path matching any of these expressions will be filtered"
-    , "out during `darcs add', or when the `--look-for-adds' flag is passed"
-    , "to `darcs whatsnew' and `record'. The entries in "
+    , "out during `darcs add`, or when the `--look-for-adds` flag is passed"
+    , "to `darcs whatsnew` and `record`. The entries in "
         ++ globalPrefsDirDoc ++ "boring (if"
     , "it exists) supplement those in this file."
     , ""
@@ -252,18 +249,6 @@ globalCacheDir | windows   = ((</> "cache2") `fmap`) `fmap` globalPrefsDir
                | osx       = ((</> "darcs") `fmap`) `fmap` osxCacheDir
                | otherwise = ((</> "darcs") `fmap`) `fmap` xdgCacheDir
 
--- |oldGlobalCacheDir is the old cache path @~/.darcs/cache@
--- now ony used with read-only access.
-oldGlobalCacheDir :: IO (Maybe FilePath)
-oldGlobalCacheDir
-    = do dir <- ((</> "cache") `fmap`) `fmap` globalPrefsDir
-         case dir of
-           Nothing -> return Nothing
-           Just d  -> do exists <- doesDirectoryExist d
-                         if exists
-                          then return $ Just d
-                          else return Nothing
-
 -- |tryMakeBoringRegexp attempts to create a Regex from a given String. The
 -- evaluation is forced, to ensure any malformed exceptions are thrown here,
 -- and not later.
@@ -273,7 +258,7 @@ tryMakeBoringRegexp input = regex `C.catch` handleBadRegex
     regex = C.evaluate (Just $! mkRegex input)
 
     handleBadRegex :: C.SomeException -> IO (Maybe Regex)
-    handleBadRegex _ = hPutDocLn Encode stderr warning >> return Nothing
+    handleBadRegex _ = hPutDocLn stderr warning >> return Nothing
 
     warning = text $ "Warning: Ignored invalid boring regex: " ++ input
 
@@ -302,10 +287,10 @@ noncomments = filter nonComment
     nonComment _ = True
 
 getPrefLines :: FilePath -> IO [String]
-getPrefLines f = removeCRsCommentsAndConflicts `fmap` readBinFile f
+getPrefLines f = removeCRsCommentsAndConflicts `fmap` readTextFile f
   where
     removeCRsCommentsAndConflicts =
-        filter notconflict . noncomments . map stripCr . lines
+        filter notconflict . noncomments . map stripCr
     startswith [] _ = True
     startswith (x : xs) (y : ys) = x == y && startswith xs ys
     startswith _ _ = False
@@ -325,7 +310,6 @@ data FileType = BinaryFile
               | TextFile
               deriving (Eq)
 
-{-# NOINLINE defaultBinaries #-}
 -- | The lines that will be inserted into @_darcs/prefs/binaries@ when
 -- @darcs init@ is run.  Hence, a list of comments, blank lines and
 -- regular expressions (ERE dialect).
@@ -333,7 +317,7 @@ data FileType = BinaryFile
 -- Note that while this matches .gz and .GZ, it will not match .gZ,
 -- i.e. it is not truly case insensitive.
 defaultBinaries :: [String]
-defaultBinaries = map ("# "++) binariesFileHelp ++
+defaultBinaries = map ("# "++) binariesFileInternalHelp ++
     [ "\\." ++ regexToMatchOrigOrUpper e ++ "$" | e <- extensions ]
   where
     regexToMatchOrigOrUpper e = "(" ++ e ++ "|" ++ map toUpper e ++ ")"
@@ -363,8 +347,8 @@ defaultBinaries = map ("# "++) binariesFileHelp ++
         , "zip"
         ]
 
-binariesFileHelp :: [String]
-binariesFileHelp =
+binariesFileInternalHelp :: [String]
+binariesFileInternalHelp =
     [ "This file contains a list of extended regular expressions, one per"
     , "line.  A file path matching any of these expressions is assumed to"
     , "contain binary data (not text). The entries in "
@@ -402,7 +386,7 @@ addToPreflist pref value = withPrefsDirectory $ \prefs -> do
     hasprefs <- doesDirectoryExist prefs
     unless hasprefs $ createDirectory prefs
     pl <- getPreflist pref
-    writeBinFile (prefs ++ pref) . unlines $ union [value] pl
+    writeTextFile (prefs ++ pref) . unlines $ union [value] pl
 
 getPreflist :: String -> IO [String]
 getPreflist p = findPrefsDirectory >>=
@@ -417,14 +401,14 @@ setPreflist :: String -> [String] -> IO ()
 setPreflist p ls = withPrefsDirectory $ \prefs -> do
     haspref <- doesDirectoryExist prefs
     when haspref $
-        writeBinFile (prefs ++ p) (unlines ls)
+        writeTextFile (prefs ++ p) (unlines ls)
 
 defPrefval :: String -> String -> IO String
 defPrefval p d = fromMaybe d `fmap` getPrefval p
 
 getPrefval :: String -> IO (Maybe String)
 getPrefval p = do
-    pl <- getPreflist "prefs"
+    pl <- getPreflist prefsDir
     return $ case map snd $ filter ((== p) . fst) $ map (break (== ' ')) pl of
                  [val] -> case words val of
                     [] -> Nothing
@@ -433,8 +417,8 @@ getPrefval p = do
 
 setPrefval :: String -> String -> IO ()
 setPrefval p v = do
-    pl <- getPreflist "prefs"
-    setPreflist "prefs" $ updatePrefVal pl p v
+    pl <- getPreflist prefsDir
+    setPreflist prefsDir $ updatePrefVal pl p v
 
 updatePrefVal :: [String] -> String -> String -> [String]
 updatePrefVal prefList p newVal =
@@ -442,10 +426,10 @@ updatePrefVal prefList p newVal =
 
 changePrefval :: String -> String -> String -> IO ()
 changePrefval p f t = do
-    pl <- getPreflist "prefs"
+    pl <- getPreflist prefsDir
     ov <- getPrefval p
     let newval = maybe t (\old -> if old == f then t else old) ov
-    setPreflist "prefs" $ updatePrefVal pl p newval
+    setPreflist prefsDir $ updatePrefVal pl p newval
 
 fixRepoPath :: String -> IO FilePath
 fixRepoPath p
@@ -513,12 +497,7 @@ getCaches useCache repodir = do
              `fmap`
              (gzFetchFilePS (repodir </> sourcesFile) Cachable
               `catchall` return B.empty)
-    oldGlobalcachedir <- oldGlobalCacheDir
     globalcachedir <- globalCacheDir
-    let oldGlobalcache = if nocache then []
-                        else case oldGlobalcachedir of
-                              Nothing -> []
-                              Just d -> [Cache Directory NotWritable d]
     let globalcache = if nocache
                           then []
                           else case globalcachedir of
@@ -530,7 +509,6 @@ getCaches useCache repodir = do
         thatrepo = [Cache Repo NotWritable repodir]
         tempCache = nub $ thisrepo ++ globalcache ++ globalsources ++ here
                           ++ thatrepo ++ filterExternalSources there
-                          ++ oldGlobalcache
     return $ Ca $ sortBy compareByLocality tempCache
   where
     sourcesFile = darcsdir ++ "/prefs/sources"
@@ -551,3 +529,158 @@ getCaches useCache repodir = do
         if isValidLocalPath repodir
             then there
             else filter (not . isValidLocalPath . cacheSource) there
+
+-- | Fetch and return the message of the day for a given repository.
+getMotd :: String -> IO B.ByteString
+getMotd repo = fetchFilePS motdPath (MaxAge 600) `catchall` return B.empty
+  where
+    motdPath = repo ++ "/" ++ darcsdir ++ "/prefs/motd"
+
+-- | Display the message of the day for a given repository,
+showMotd :: String -> IO ()
+showMotd repo = do
+    motd <- getMotd repo
+    unless (B.null motd) $ do
+        B.hPut stdout motd
+        putStrLn $ replicate 22 '*'
+
+prefsUrl :: FilePath -> String
+prefsUrl r = r ++ "/"++darcsdir++"/prefs"
+
+prefsDir :: FilePath
+prefsDir = "prefs"
+
+prefsDirPath :: FilePath
+prefsDirPath = darcsdir </> prefsDir
+
+prefsFilesHelp :: [(String,String)]
+prefsFilesHelp  =
+    [ ("motd", unlines
+      [ "The `_darcs/prefs/motd` file may contain a 'message of the day' which"
+      , "will be displayed to users who clone or pull from the repository without"
+      , "the `--quiet` option."])
+    , ("email", unlines
+      [ "The `_darcs/prefs/email` file is used to provide the e-mail address for"
+      , "your repository that others will use when they `darcs send` a patch back"
+      , "to you. The contents of the file should simply be an e-mail address."])
+    , ("post", unlines
+      [ "If `_darcs/prefs/post` exists in the target repository, `darcs send ` will"
+      , "upload to the URL contained in that file, which may either be a `mailto:`"
+      , "URL, or an `http://` URL. In the latter case, the patch is posted to that URL."])
+    , ("author", unlines
+      [ "The `_darcs/prefs/author` file contains the email address (or name) to"
+      , "be used as the author when patches are recorded in this repository,"
+      , "e.g. `David Roundy <droundy@abridgegame.org>`. This file overrides the"
+      , "contents of the environment variables `$DARCS_EMAIL` and `$EMAIL`."])
+    , ("defaults", unlines
+      [ "Default values for darcs commands. Each line of this file has the"
+      , "following form:"
+      , ""
+      , "    COMMAND FLAG VALUE"
+      , ""
+      , "where `COMMAND` is either the name of the command to which the default"
+      , "applies, or `ALL` to indicate that the default applies to all commands"
+      , "accepting that flag. The `FLAG` term is the name of the long argument"
+      , "option without the `--`, i.e. `verbose` rather than `--verbose`."
+      , "Finally, the `VALUE` option can be omitted if the flag does not involve"
+      , "a value. If the value has spaces in it, use single quotes, not double"
+      , "quotes, to surround it. Each line only takes one flag. To set multiple"
+      , "defaults for the same command (or for `ALL` commands), use multiple lines."
+      , ""
+      , "Note that the use of `ALL` easily can have unpredicted consequences,"
+      , "especially if commands in newer versions of darcs accepts flags that"
+      , "they did not in previous versions. Only use safe flags with `ALL`."
+      , ""
+      , "For example, if your system clock is bizarre, you could instruct darcs to"
+      , "always ignore the file modification times by adding the following line:"
+      , ""
+      , "    ALL ignore-times"
+      , ""
+      , "There are some options which are meant specifically for use in"
+      , "`_darcs/prefs/defaults`. One of them is `--disable`. As the name"
+      , "suggests, this option will disable every command that got it as"
+      , "argument. So, if you are afraid that you could damage your repositories"
+      , "by inadvertent use of a command like amend, add the following line:"
+      , ""
+      , "    amend disable"
+      , ""
+      , "Also, a global preferences file can be created with the name"
+      , "`.darcs/defaults` in your home directory. Options present there will be"
+      , "added to the repository-specific preferences if they do not conflict."])
+    , ("sources", unlines
+      [ "The `_darcs/prefs/sources` file is used to indicate alternative locations"
+      , "from which to download patches. This file contains lines such as:"
+      , ""
+      , "    cache:/home/droundy/.cache/darcs"
+      , "    readonly:/home/otheruser/.cache/darcs"
+      , "    repo:http://darcs.net"
+      , ""
+      , "This would indicate that darcs should first look in"
+      , "`/home/droundy/.cache/darcs` for patches that might be missing, and if"
+      , "the patch is not there, it should save a copy there for future use."
+      , "In that case, darcs will look in `/home/otheruser/.cache/darcs` to see if"
+      , "that user might have downloaded a copy, but will not try to save a copy"
+      , "there, of course. Finally, it will look in `http://darcs.net`. Note that"
+      , "the `sources` file can also exist in `~/.darcs/`. Also note that the"
+      , "sources mentioned in your `sources` file will be tried *before* the"
+      , "repository you are pulling from. This can be useful in avoiding"
+      , "downloading patches multiple times when you pull from a remote"
+      , "repository to more than one local repository."
+      , ""
+      , "A global cache is enabled by default in your home directory. The cache"
+      , "allows darcs to avoid re-downloading patches (for example, when doing a"
+      , "second darcs clone of the same repository), and also allows darcs to use"
+      , "hard links to reduce disk usage."
+      , ""
+      , "Note that the cache directory should reside on the same filesystem as"
+      , "your repositories, so you may need to vary this. You can also use"
+      , "multiple cache directories on different filesystems, if you have several"
+      , "filesystems on which you use darcs."])
+    , ("boring", unlines
+      [ "The `_darcs/prefs/boring` file may contain a list of regular expressions"
+      , "describing files, such as object files, that you do not expect to add to"
+      , "your project. A newly created repository has a boring file that includes"
+      , "many common source control, backup, temporary, and compiled files."
+      , ""
+      , "You may want to have the boring file under version control. To do this"
+      , "you can use darcs setpref to set the value 'boringfile' to the name of"
+      , "your desired boring file (e.g. `darcs setpref boringfile .boring`, where"
+      , "`.boring` is the repository path of a file that has been darcs added to"
+      , "your repository). The boringfile preference overrides"
+      , "`_darcs/prefs/boring`, so be sure to copy that file to the boringfile."
+      , ""
+      , "You can also set up a 'boring' regexps file in your home directory, named"
+      , "`~/.darcs/boring`, which will be used with all of your darcs repositories."
+      , ""
+      , "Any file not already managed by darcs and whose repository path"
+      , "matches any of the boring regular expressions is"
+      , "considered boring. The boring file is used to filter the files provided"
+      , "to darcs add, to allow you to use a simple `darcs add newdir newdir/*`"
+      , "without accidentally adding a bunch of object files. It is also used"
+      , "when the `--look-for-adds` flag is given to whatsnew or record. Note"
+      , "that once a file has been added to darcs, it is not considered boring,"
+      , "even if it matches the boring file filter."])
+    , ("binaries", unlines
+      [ "The `_darcs/prefs/binaries` file may contain a list of regular"
+      , "expressions describing files that should be treated as binary files rather"
+      , "than text files. Darcs automatically treats files containing characters"
+      , "`^Z` or `NULL` within the first 4096 bytes as being binary files."
+      , "You probably will want to have the binaries file under version control."
+      , "To do this you can use `darcs setpref` to set the value 'binariesfile'"
+      , "to the name of your desired binaries file"
+      , "(e.g. `darcs setpref binariesfile ./.binaries`, where `.binaries` is a"
+      , "file that has been darcs added to your repository). As with the boring"
+      , "file, you can also set up a `~/.darcs/binaries` file if you like."])
+    , ("defaultrepo", unlines
+      [ "Contains the URL of the default remote repository used by commands `pull`,"
+      , "`push`, `send` and `optimize relink`. Darcs edits this file automatically"
+      , "or when the flag `--set-default` is used."])
+    , ("tmpdir", unlines
+      [ "By default temporary directories are created in `/tmp`, or if that doesn't"
+      , "exist, in `_darcs` (within the current repo).  This can be overridden by"
+      , "specifying some other directory in the file `_darcs/prefs/tmpdir` or the"
+      , "environment variable `$DARCS_TMPDIR` or `$TMPDIR`."])
+    , ("prefs", unlines
+      [ "Contains the preferences set by the command `darcs setprefs`."
+      , "Do not edit manually."])
+    ]
