@@ -43,8 +43,9 @@ import Darcs.UI.Commands
     , amInHashedRepository
     )
 import Darcs.UI.Commands.Util ( announceFiles, testTentativeAndMaybeExit )
+import Darcs.UI.Completion ( modifiedFileArgs, knownFileArgs )
 import Darcs.UI.Flags ( DarcsFlag, diffOpts, fixSubPaths )
-import Darcs.UI.Options ( DarcsOption, (^), oparse, odesc, ocheck, defaultFlags )
+import Darcs.UI.Options ( DarcsOption, (^), oparse, odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.PatchHeader ( updatePatchHeader, AskAboutDeps(..)
                             , HijackOptions(..)
@@ -70,12 +71,10 @@ import Darcs.Repository
     , withManualRebaseUpdate
     , finalizeRepositoryChanges
     , invalidateIndex
-    , unrecordedChangesWithPatches
+    , unrecordedChanges
     , readRecorded
-    , listRegisteredFiles
     )
 import Darcs.Repository.Prefs ( globalPrefsDirDoc )
-import Darcs.Repository.State ( getMovesPs, getReplaces )
 import Darcs.UI.SelectChanges
     ( WhichChanges(..)
     , selectionContextPrim
@@ -88,12 +87,10 @@ import qualified Darcs.UI.SelectChanges as S
 import Darcs.Util.Exception ( clarifyErrors )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), (:>)(..), (+>+), nullFL, reverseRL, mapFL_FL )
-import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
-import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 
 import Darcs.Util.Printer ( putDocLn )
 import Darcs.Util.Tree( Tree )
-import Darcs.Repository.Internal ( tentativelyRemoveFromPending )
+import Darcs.Repository.Pending ( tentativelyRemoveFromPending )
 
 
 amendDescription :: String
@@ -151,11 +148,11 @@ amendBasicOpts
     ^ O.author
     ^ O.selectAuthor
     ^ O.patchname
-    ^ O.askdeps
+    ^ O.askDeps
     ^ O.askLongComment
     ^ O.keepDate
     ^ O.lookfor
-    ^ O.workingRepoDir
+    ^ O.repoDir
     ^ O.withContext
     ^ O.diffAlgorithm
 
@@ -192,10 +189,7 @@ amendOpts :: DarcsOption a
               -> O.UMask
               -> O.SetScriptsExecutable
               -> O.UseCache
-              -> Maybe String
-              -> Bool
-              -> Maybe String
-              -> Bool
+              -> O.HooksConfig
               -> a)
 amendOpts = withStdOpts amendBasicOpts amendAdvancedOpts
 
@@ -237,7 +231,7 @@ amend = DarcsCommand
     , commandExtraArgHelp         = ["[FILE or DIRECTORY]..."]
     , commandCommand              = amendCmd
     , commandPrereq               = amInHashedRepository
-    , commandGetArgPossibilities  = listRegisteredFiles
+    , commandCompleteArgs         = amendFileArgs
     , commandArgdefaults          = nodefaults
     , commandAdvancedOptions      = odesc amendAdvancedOpts
     , commandBasicOptions         = odesc amendBasicOpts
@@ -245,7 +239,11 @@ amend = DarcsCommand
     , commandCheckOptions         = ocheck amendOpts
     , commandParseOptions         = amendConfig
     }
-
+  where
+    amendFileArgs fps flags args =
+      if (O.amendUnrecord ? flags)
+        then knownFileArgs fps flags args
+        else modifiedFileArgs fps flags args
 
 amendrecord :: DarcsCommand AmendConfig
 amendrecord = commandAlias "amend-record" Nothing amend
@@ -293,15 +291,9 @@ doAmend cfg files =
                            (_ :> chosenPrims) <- runSelection (effect oldp) context
                            let invPrims = reverseRL (invertFL chosenPrims)
                            addChangesToPatch cfg repository oldp invPrims
-                   else do Sealed replacePs <- if O.replaces (lookfor cfg) == O.YesLookForReplaces
-                             then getReplaces (diffingOpts cfg) repository files
-                             else return (Sealed NilFL)
-                           movesPs <- if O.moves (lookfor cfg) == O.YesLookForMoves
-                              then getMovesPs repository files
-                              else return NilFL
-                           go =<< unrecordedChangesWithPatches
-                                    movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
-                                    (diffingOpts cfg) repository files
+                   else go =<< unrecordedChanges (diffingOpts cfg)
+                                  (O.moves (lookfor cfg)) (O.replaces (lookfor cfg))
+                                  repository files
               -- amending a tag
            else if hasEditMetadata cfg && isNothing files
                         -- the user is not trying to add new changes to the tag so there is
