@@ -15,7 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, MagicHash, OverloadedStrings #-}
+{-# LANGUAGE MagicHash, OverloadedStrings #-}
 
 module Darcs.UI.Commands.Convert ( convert ) where
 
@@ -23,8 +23,11 @@ import Prelude ( lookup )
 import Darcs.Prelude hiding ( readFile, lex )
 
 import System.FilePath.Posix ( (</>) )
-import System.Directory ( setCurrentDirectory, doesDirectoryExist, doesFileExist,
-                   createDirectory, removeFile )
+import System.Directory
+    ( doesDirectoryExist
+    , doesFileExist
+    , removeFile
+    )
 import System.IO ( stdin )
 import Data.IORef ( newIORef, modifyIORef, readIORef )
 import Data.Char ( isSpace )
@@ -35,7 +38,6 @@ import Control.Monad.State.Strict ( gets, modify )
 import Control.Exception ( finally )
 import Control.Applicative ( (<|>) )
 
-import GHC.Base ( unsafeCoerce# )
 import System.Time ( toClockTime )
 import Data.Maybe ( catMaybes, fromMaybe )
 import qualified Data.IntMap as M
@@ -49,6 +51,7 @@ import qualified Data.ByteString.Lazy.UTF8 as BLU
 import qualified Data.Attoparsec.ByteString.Char8 as A
 import Data.Attoparsec.ByteString.Char8( (<?>) )
 
+import Darcs.Util.ByteString ( decodeLocale )
 import qualified Darcs.Util.Tree as T
 import qualified Darcs.Util.Tree.Monad as TM
 import Darcs.Util.Tree.Monad hiding ( createDirectory, exists, rename )
@@ -57,14 +60,14 @@ import Darcs.Util.Tree( Tree, treeHash, readBlob, TreeItem(..)
                       , emptyTree, listImmediate, findTree )
 import Darcs.Util.Path( anchorPath, appendPath, floatPath
                       , parent, anchoredRoot
-                      , AnchoredPath(..), Name(..)
+                      , AnchoredPath(..), makeName
                       , ioAbsoluteOrRemote, toPath, AbsolutePath )
 import Darcs.Util.Hash( encodeBase16, sha256, Hash(..) )
 
 import Darcs.Util.DateTime ( formatDateTime, fromClockTime, parseDateTime, startOfTime )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Exception ( clarifyErrors )
+import Darcs.Util.Lock ( withNewDirectory )
 import Darcs.Util.Prompt ( askUser )
 import Darcs.Util.Printer ( text, ($$) )
 import Darcs.Util.Printer.Color ( traceDoc )
@@ -72,10 +75,9 @@ import Darcs.Util.Workaround ( getCurrentDirectory )
 
 import Darcs.Patch.Depends ( getUncovered )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia, info, hopefully )
-import Darcs.Patch ( IsRepoType, showPatch, fromPrim, fromPrims,
-                     effect,
-                     RepoPatch, apply, listTouchedFiles
-                   , move )
+import Darcs.Patch
+    ( showPatch, ShowPatchFor(..), fromPrim, fromPrims
+    , effect, RepoPatch, apply, listTouchedFiles, move )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Effect ( Effect )
 import Darcs.Patch.Named
@@ -86,62 +88,73 @@ import Darcs.Patch.Named.Wrapped ( WrappedNamed(..) )
 import qualified Darcs.Patch.Named.Wrapped as Wrapped ( getdeps )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..), (=/\=) )
 import Darcs.Patch.Witnesses.Ordered
-    ( FL(..), RL(..), bunchFL, mapFL, mapFL_FL,
+    ( FL(..), RL(..), bunchFL, mapFL_FL,
     concatFL, mapRL, nullFL, (+>+), (+<+)
-    , reverseRL, reverseFL )
+    , reverseRL, reverseFL, foldFL_M )
 import Darcs.Patch.Witnesses.Sealed ( FlippedSeal(..), Sealed(..), unFreeLeft
-                                    ,  flipSeal, unsafeUnsealFlipped )
+                                    , mapSeal, flipSeal, unsafeUnsealFlipped )
+
 import Darcs.Patch.Info ( piRename, piTag, isTag, PatchInfo, patchinfo,
                           piName, piLog, piDate, piAuthor, makePatchname )
-import Darcs.Patch.V1 ( RepoPatchV1 )
-import Darcs.Patch.V2 ( RepoPatchV2 )
+import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
+import qualified Darcs.Patch.V2 as V2 ( RepoPatchV2 )
+import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
 import Darcs.Patch.V1.Commute ( publicUnravel )
-import Darcs.Patch.V1.Core ( RepoPatchV1(PP), isMerger )
+import qualified Darcs.Patch.V1.Core as V1 ( RepoPatchV1(PP), isMerger )
 import Darcs.Patch.V2.RepoPatch ( mergeUnravelled )
 import Darcs.Patch.Prim ( sortCoalesceFL )
 import Darcs.Patch.Prim.Class ( PrimOf )
-import Darcs.Patch.Prim.V1 ( Prim )
-import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
-import Darcs.Patch.Set ( PatchSet(..), Tagged(..), newset2RL, newset2FL )
+import Darcs.Patch.RepoType ( RepoType(..), IsRepoType(..), RebaseType(..) )
+import Darcs.Patch.Set ( PatchSet(..), Tagged(..), patchSet2RL, patchSet2FL )
 import Darcs.Patch.Progress ( progressFL )
 
-import Darcs.Repository.Flags ( UpdateWorking(..), Reorder (..), UseIndex(..), ScanKnown(..)
-                              , AllowConflicts(..), ExternalMerge(..), WantGuiPause(..), PatchFormat(..)
-                              , Compression(..), DryRun(NoDryRun), DiffAlgorithm(MyersDiff, PatienceDiff) )
-import Darcs.Repository ( Repository, withRepoLock, RepoJob(..), withRepositoryDirectory,
-                          createRepository, invalidateIndex,
-                          tentativelyMergePatches,
-                          createPristineDirectoryTree,
-                          revertRepositoryChanges, finalizeRepositoryChanges,
-                          applyToWorking
-                        , readRepo, readTentativeRepo, cleanRepository )
+import Darcs.Repository.Flags
+    ( UpdateWorking(..)
+    , Compression(..)
+    , DiffAlgorithm(PatienceDiff) )
+import Darcs.Repository
+    ( Repository, RepoJob(..), withRepositoryLocation
+    , createRepository, invalidateIndex, repoLocation
+    , createPristineDirectoryTree, repoCache
+    , revertRepositoryChanges, finalizeRepositoryChanges
+    , applyToWorking, repoLocation, repoCache
+    , readRepo, readTentativeRepo, cleanRepository
+    , createRepositoryV2, EmptyRepository(..)
+    , withUMaskFlag
+    )
 import qualified Darcs.Repository as R( setScriptsExecutable )
+import Darcs.Repository.InternalTypes ( coerceR )
 import Darcs.Repository.State( readRecorded )
 import Darcs.Repository.Cache ( HashedDir( HashedPristineDir ) )
-import Darcs.Repository.InternalTypes ( extractCache )
-import Darcs.Repository.HashedRepo ( readHashedPristineRoot, addToTentativeInventory )
+import Darcs.Repository.Hashed
+    ( tentativelyAddPatch_
+    , UpdatePristine(..)
+    , readHashedPristineRoot
+    , addToTentativeInventory )
 import Darcs.Repository.HashedIO ( cleanHashdir )
-import Darcs.Repository.Prefs( FileType(..) )
+import Darcs.Repository.Prefs( FileType(..), showMotd )
 import Darcs.Repository.Format(identifyRepoFormat, formatHas, RepoProperty(Darcs2))
-import Darcs.Repository.Motd ( showMotd )
-import Darcs.Util.Lock ( writeBinFile )
 import Darcs.Util.External ( fetchFilePS, Cachable(Uncachable) )
 import Darcs.Repository.Diff( treeDiff )
 
 
 import Darcs.UI.External ( catchall )
 import Darcs.UI.Flags
-    ( verbosity, useCache, umask, withWorkingDir, runPatchIndex
+    ( verbosity, useCache, umask, withWorkingDir, patchIndexNo
     , DarcsFlag ( NewRepo )
-    , getRepourl, patchFormat
+    , getRepourl, patchFormat, quiet
     )
 import Darcs.UI.Commands ( DarcsCommand(..), amInRepository, nodefaults, putInfo
                          , normalCommand, withStdOpts )
 import Darcs.UI.Commands.Util.Tree ( treeHasDir, treeHasFile )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+import Darcs.UI.Completion ( noArgs )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 
-#include "impossible.h"
+type RepoPatchV1 = V1.RepoPatchV1 V1.Prim
+type RepoPatchV2 = V2.RepoPatchV2 V2.Prim
+
 
 convertDescription :: String
 convertDescription = "Convert repositories between various formats."
@@ -212,46 +225,19 @@ convertImportHelp = unlines
  ]
 
 convert :: DarcsCommand [DarcsFlag]
-convert = SuperCommand {
-      commandProgramName = "darcs"
+convert = SuperCommand
+    { commandProgramName = "darcs"
     , commandName = "convert"
     , commandHelp = ""
     , commandDescription = convertDescription
     , commandPrereq = amInRepository
-    , commandSubCommands = [  normalCommand convertDarcs2,
-                              normalCommand convertExport,
-                              normalCommand convertImport
-                           ]
+    , commandSubCommands =
+        [ normalCommand convertDarcs2
+        , normalCommand convertExport
+        , normalCommand convertImport
+        ]
     }
 
-convertDarcs2BasicOpts :: DarcsOption a (Maybe String -> O.SetScriptsExecutable -> O.WithWorkingDir -> a)
-convertDarcs2BasicOpts = O.reponame ^ O.setScriptsExecutable ^ O.useWorkingDir
-
-convertDarcs2AdvancedOpts :: DarcsOption a (O.NetworkOptions -> O.WithPatchIndex -> a)
-convertDarcs2AdvancedOpts = O.network ^ O.patchIndex
-
-convertDarcs2Opts :: DarcsOption a
-                     (Maybe String
-                      -> O.SetScriptsExecutable
-                      -> O.WithWorkingDir
-                      -> Maybe O.StdCmdAction
-                      -> Bool
-                      -> Bool
-                      -> O.Verbosity
-                      -> Bool
-                      -> O.NetworkOptions
-                      -> O.WithPatchIndex
-                      -> O.UseCache
-                      -> Maybe String
-                      -> Bool
-                      -> Maybe String
-                      -> Bool
-                      -> a)
-convertDarcs2Opts = convertDarcs2BasicOpts `withStdOpts` convertDarcs2AdvancedOpts
-
-convertDarcs2SilentOpts :: DarcsOption a (O.PatchFormat -> a)
-convertDarcs2SilentOpts = O.patchFormat
-
 convertDarcs2 :: DarcsCommand [DarcsFlag]
 convertDarcs2 = DarcsCommand
     { commandProgramName = "darcs"
@@ -262,7 +248,7 @@ convertDarcs2 = DarcsCommand
     , commandExtraArgHelp = ["<SOURCE>", "[<DESTINATION>]"]
     , commandCommand = toDarcs2
     , commandPrereq = \_ -> return $ Right ()
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc convertDarcs2AdvancedOpts
     , commandBasicOptions = odesc convertDarcs2BasicOpts
@@ -270,31 +256,11 @@ convertDarcs2 = DarcsCommand
     , commandCheckOptions = ocheck convertDarcs2Opts
     , commandParseOptions = onormalise convertDarcs2Opts
     }
-
-convertExportBasicOpts :: DarcsOption a
-                          (Maybe String -> Maybe String -> Maybe String -> a)
-convertExportBasicOpts = O.reponame ^ O.marks
-
-convertExportAdvancedOpts :: DarcsOption a (O.NetworkOptions -> a)
-convertExportAdvancedOpts = O.network
-
-convertExportOpts :: DarcsOption a
-                     (Maybe String
-                      -> Maybe String
-                      -> Maybe String
-                      -> Maybe O.StdCmdAction
-                      -> Bool
-                      -> Bool
-                      -> O.Verbosity
-                      -> Bool
-                      -> O.NetworkOptions
-                      -> O.UseCache
-                      -> Maybe String
-                      -> Bool
-                      -> Maybe String
-                      -> Bool
-                      -> a)
-convertExportOpts = convertExportBasicOpts `withStdOpts` convertExportAdvancedOpts
+  where
+    convertDarcs2BasicOpts = O.reponame ^ O.setScriptsExecutable ^ O.withWorkingDir
+    convertDarcs2AdvancedOpts = O.network ^ O.patchIndexNo
+    convertDarcs2Opts = convertDarcs2BasicOpts `withStdOpts` convertDarcs2AdvancedOpts
+    convertDarcs2SilentOpts = O.patchFormat
 
 convertExport :: DarcsCommand [DarcsFlag]
 convertExport = DarcsCommand
@@ -306,7 +272,7 @@ convertExport = DarcsCommand
     , commandExtraArgHelp = []
     , commandCommand = fastExport
     , commandPrereq = amInRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc convertExportAdvancedOpts
     , commandBasicOptions = odesc convertExportBasicOpts
@@ -314,40 +280,10 @@ convertExport = DarcsCommand
     , commandCheckOptions = ocheck convertExportOpts
     , commandParseOptions = onormalise convertExportOpts
     }
-
-convertImportBasicOpts :: DarcsOption a
-                          (Maybe String
-                           -> O.SetScriptsExecutable
-                           -> O.PatchFormat
-                           -> O.WithWorkingDir
-                           -> a)
-convertImportBasicOpts
-  = O.reponame
-  ^ O.setScriptsExecutable
-  ^ O.patchFormat
-  ^ O.useWorkingDir
-
-convertImportAdvancedOpts :: DarcsOption a (O.WithPatchIndex -> a)
-convertImportAdvancedOpts = O.patchIndex
-
-convertImportOpts :: DarcsOption a
-                     (Maybe String
-                      -> O.SetScriptsExecutable
-                      -> O.PatchFormat
-                      -> O.WithWorkingDir
-                      -> Maybe O.StdCmdAction
-                      -> Bool
-                      -> Bool
-                      -> O.Verbosity
-                      -> Bool
-                      -> O.WithPatchIndex
-                      -> O.UseCache
-                      -> Maybe String
-                      -> Bool
-                      -> Maybe String
-                      -> Bool
-                      -> a)
-convertImportOpts = convertImportBasicOpts `withStdOpts` convertImportAdvancedOpts
+  where
+    convertExportBasicOpts = O.reponame ^ O.marks
+    convertExportAdvancedOpts = O.network
+    convertExportOpts = convertExportBasicOpts `withStdOpts` convertExportAdvancedOpts
 
 convertImport :: DarcsCommand [DarcsFlag]
 convertImport = DarcsCommand
@@ -359,7 +295,7 @@ convertImport = DarcsCommand
     , commandExtraArgHelp = ["[<DIRECTORY>]"]
     , commandCommand = fastImport
     , commandPrereq = \_ -> return $ Right ()
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc convertImportAdvancedOpts
     , commandBasicOptions = odesc convertImportBasicOpts
@@ -367,15 +303,25 @@ convertImport = DarcsCommand
     , commandCheckOptions = ocheck convertImportOpts
     , commandParseOptions = onormalise convertImportOpts
     }
+  where
+    convertImportBasicOpts
+      = O.reponame
+      ^ O.setScriptsExecutable
+      ^ O.patchFormat
+      ^ O.withWorkingDir
+    convertImportAdvancedOpts = O.patchIndexNo
+    convertImportOpts = convertImportBasicOpts `withStdOpts` convertImportAdvancedOpts
 
 toDarcs2 :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-toDarcs2 fps opts [inrepodir, outname] = toDarcs2 fps (NewRepo outname:opts) [inrepodir]
-toDarcs2 _ opts [inrepodir] = do
-
+toDarcs2 _ opts' args = do
+  (inrepodir, opts) <-
+    case args of
+      [arg1, arg2] -> return (arg1, NewRepo arg2:opts')
+      [arg1] -> return (arg1, opts')
+      _ -> fail "You must provide either one or two arguments."
   typed_repodir <- ioAbsoluteOrRemote inrepodir
   let repodir = toPath typed_repodir
 
-  --test for converting darcs-2 repository
   format <- identifyRepoFormat repodir
   when (formatHas Darcs2 format) $ fail "Repository is already in darcs 2 format."
 
@@ -385,42 +331,46 @@ toDarcs2 _ opts [inrepodir] = do
   vow' <- askUser ("by typing `" ++ vow ++ "': ")
   when (vow' /= vow) $ fail "User didn't understand the consequences."
 
-  unless (parseFlags O.verbosity opts == O.Quiet) $ showMotd repodir
+  unless (quiet opts) $ showMotd repodir
+
   mysimplename <- makeRepoName opts repodir
-  createDirectory mysimplename
-  setCurrentDirectory mysimplename
-  createRepository PatchFormat2 (withWorkingDir opts) (runPatchIndex opts)
-  writeBinFile (darcsdir++"/hashed_inventory") ""
-  withRepoLock NoDryRun (useCache opts) NoUpdateWorking (umask opts) $ V2Job $ \repository ->
-    withRepositoryDirectory (useCache opts) repodir $ V1Job $ \themrepo -> do
-      theirstuff <- readRepo themrepo
-      let patches = mapFL_FL (convertNamed . hopefully) $ newset2FL theirstuff
-          outOfOrderTags = catMaybes $ mapRL oot $ newset2RL theirstuff
+  withUMaskFlag (umask ? opts) $ withNewDirectory mysimplename $ do
+    repo <- createRepositoryV2
+      (withWorkingDir ? opts) (patchIndexNo ? opts) (O.useCache ? opts)
+    revertRepositoryChanges repo NoUpdateWorking
+
+    withRepositoryLocation (useCache ? opts) repodir $ V1Job $ \other -> do
+      theirstuff <- readRepo other
+      let patches = mapFL_FL (convertNamed . hopefully) $ patchSet2FL theirstuff
+          outOfOrderTags = catMaybes $ mapRL oot $ patchSet2RL theirstuff
               where oot t = if isTag (info t) && info t `notElem` inOrderTags theirstuff
                             then Just (info t, Wrapped.getdeps $ hopefully t)
                             else Nothing
           fixDep p = case lookup p outOfOrderTags of
                      Just d -> p : concatMap fixDep d
                      Nothing -> [p]
-          convertOne :: RepoPatchV1 Prim wX wY -> FL (RepoPatchV2 Prim) wX wY
-          convertOne x | isMerger x = case mergeUnravelled $ publicUnravel x of
-                                       Just (FlippedSeal y) ->
-                                           case effect y =/\= effect x of
-                                           IsEq -> y :>: NilFL
-                                           NotEq ->
-                                               traceDoc (text "lossy conversion:" $$
-                                                         showPatch x)
-                                               fromPrims (effect x)
-                                       Nothing -> traceDoc (text
-                                                            "lossy conversion of complicated conflict:" $$
-                                                            showPatch x)
-                                                  fromPrims (effect x)
-          convertOne (PP x) = fromPrim x :>: NilFL
+          primV1toV2 = V2.Prim . V1.unPrim
+          convertOne :: RepoPatchV1 wX wY -> FL RepoPatchV2 wX wY
+          convertOne x | V1.isMerger x =
+            let ex = mapFL_FL primV1toV2 (effect x) in
+            case mergeUnravelled $ map (mapSeal (mapFL_FL primV1toV2)) $ publicUnravel x of
+             Just (FlippedSeal y) ->
+                 case effect y =/\= ex of
+                 IsEq -> y :>: NilFL
+                 NotEq ->
+                     traceDoc (text "lossy conversion:" $$
+                               showPatch ForDisplay x)
+                     fromPrims ex
+             Nothing -> traceDoc (text
+                                  "lossy conversion of complicated conflict:" $$
+                                  showPatch ForDisplay x)
+                        fromPrims ex
+          convertOne (V1.PP x) = fromPrim (primV1toV2 x) :>: NilFL
           convertOne _ = impossible
-          convertFL :: FL (RepoPatchV1 Prim) wX wY -> FL (RepoPatchV2 Prim) wX wY
+          convertFL :: FL RepoPatchV1 wX wY -> FL RepoPatchV2 wX wY
           convertFL = concatFL . mapFL_FL convertOne
-          convertNamed :: WrappedNamed ('RepoType 'NoRebase) (RepoPatchV1 Prim) wX wY
-                       -> PatchInfoAnd ('RepoType 'NoRebase) (RepoPatchV2 Prim) wX wY
+          convertNamed :: WrappedNamed ('RepoType 'NoRebase) RepoPatchV1 wX wY
+                       -> PatchInfoAnd ('RepoType 'NoRebase) RepoPatchV2 wX wY
           convertNamed (NormalP n)
                          = n2pia $ NormalP $
                            adddeps (infopatch (convertInfo $ patch2patchinfo n) $
@@ -428,24 +378,9 @@ toDarcs2 _ opts [inrepodir] = do
                                    (map convertInfo $ concatMap fixDep $ getdeps n)
           convertInfo n | n `elem` inOrderTags theirstuff = n
                         | otherwise = maybe n (\t -> piRename n ("old tag: "++t)) $ piTag n
-          applySome xs = do -- TODO this unsafeCoerce hack is because we don't keep track of the repository state properly
-                            -- Really sequence_ $ mapFL applySome below should instead be a repeated add operation -
-                            -- there doesn't seem to be any reason we need to do a merge here.
-                            let repository2 = unsafeCoerce# repository :: Repository ('RepoType 'NoRebase) (RepoPatchV2 Prim) wA wB wA
-                            Sealed pw <- tentativelyMergePatches repository2 "convert"
-                                             YesAllowConflicts NoUpdateWorking
-                                             NoExternalMerge NoWantGuiPause
-                                             GzipCompression (verbosity opts)
-                                             NoReorder
-                                             (UseIndex, ScanKnown, MyersDiff)
-                                             NilFL xs
-                            finalizeRepositoryChanges repository2 NoUpdateWorking GzipCompression -- this is to clean out pristine.hashed
-                            revertRepositoryChanges repository2 NoUpdateWorking
-                            _ <- revertable $ applyToWorking repository2 (verbosity opts) pw
-                            invalidateIndex repository2
-      sequence_ $ mapFL applySome $ bunchFL 100 $ progressFL "Converting patch" patches
-      invalidateIndex repository
-      revertable $ createPristineDirectoryTree repository "." (withWorkingDir opts)
+
+      -- Note: we use bunchFL so we can commit every 100 patches
+      _ <- applyAll opts repo $ bunchFL 100 $ progressFL "Converting patch" patches
       when (parseFlags O.setScriptsExecutable opts == O.YesSetScriptsExecutable)
         R.setScriptsExecutable
 
@@ -455,12 +390,70 @@ toDarcs2 _ opts [inrepodir] = do
        `catchall` return ()
 
       putInfo opts $ text "Finished converting."
-      where revertable x = x `clarifyErrors` unlines
-                  ["An error may have left your new working directory an inconsistent",
-                   "but recoverable state. You should be able to make the new",
-                   "repository consistent again by running darcs revert -a."]
+  where
+    applyOne :: (RepoPatch p, ApplyState p ~ Tree)
+             => [DarcsFlag]
+             -> W2 (Repository rt p wR) wX
+             -> PatchInfoAnd rt p wX wY
+             -> IO (W2 (Repository rt p wR) wY)
+    applyOne opts (W2 r) x = do
+      r' <- tentativelyAddPatch_ (updatePristine opts) r
+        GzipCompression (verbosity ? opts) (updateWorking opts) x
+      r'' <- withTryAgainMsg $ applyToWorking r' (verbosity ? opts) (effect x)
+      invalidateIndex r''
+      return (W2 r'')
 
-toDarcs2 _ _ _ = fail "You must provide either one or two arguments."
+    applySome opts (W3 r) xs = do
+      r' <- unW2 <$> foldFL_M (applyOne opts) (W2 r) xs
+      -- commit after applying a bunch of patches
+      finalizeRepositoryChanges r' (updateWorking opts) GzipCompression
+      revertRepositoryChanges r' (updateWorking opts)
+      -- finalizeRepositoryChanges and revertRepositoryChanges
+      -- do not (yet?) return a repo with properly coerced witnesses.
+      -- We should have
+      --
+      -- > finalizeRepositoryChanges :: ... wR wU wT -> ... wT wU wT
+      --
+      -- and
+      --
+      -- > revertRepositoryChanges :: ... wR wU wT -> ... wR wU wR
+      --
+      -- This is why we must coerce here:
+      return (W3 (coerceR r'))
+
+    applyAll :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+             => [DarcsFlag]
+             -> Repository rt p wX wX wX
+             -> FL (FL (PatchInfoAnd rt p)) wX wY
+             -> IO (Repository rt p wY wY wY)
+    applyAll opts r xss = unW3 <$> foldFL_M (applySome opts) (W3 r) xss
+
+    updatePristine :: [DarcsFlag] -> UpdatePristine
+    updatePristine opts =
+      case withWorkingDir ? opts of
+        O.WithWorkingDir -> UpdatePristine
+        -- this should not be necessary but currently is, because
+        -- some commands (e.g. send) cannot cope with a missing pristine
+        -- even if the repo is marked as having no working tree
+        O.NoWorkingDir -> {- DontUpdatePristineNorRevert -}UpdatePristine
+
+    updateWorking :: [DarcsFlag] -> UpdateWorking
+    updateWorking opts =
+      case withWorkingDir ? opts of
+        O.WithWorkingDir -> YesUpdateWorking
+        O.NoWorkingDir -> NoUpdateWorking
+
+    withTryAgainMsg :: IO a -> IO a
+    withTryAgainMsg x = x `clarifyErrors` unlines
+      [ "An error occurred while applying patches to the working tree."
+      , "You may have more luck if you supply --no-working-dir." ]
+
+-- | Need this to make 'foldFL_M' work with a function that changes
+-- the last two (identical) witnesses at the same time.
+newtype W2 r wX = W2 {unW2 :: r wX wX}
+
+-- | Similarly for when the function changes all three witnesses.
+newtype W3 r wX = W3 {unW3 :: r wX wX wX}
 
 makeRepoName :: [DarcsFlag] -> FilePath -> IO String
 makeRepoName (NewRepo n:_) _ =
@@ -503,7 +496,7 @@ fastExport _ opts _ = do
   marks <- case parseFlags O.readMarks opts of
     Nothing -> return emptyMarks
     Just f  -> readMarks f
-  newMarks <- withRepositoryDirectory (useCache opts) repodir $ RepoJob $ \repo -> fastExport' repo marks
+  newMarks <- withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repo -> fastExport' repo marks
   case parseFlags O.writeMarks opts of
     Nothing -> return ()
     Just f  -> writeMarks f newMarks
@@ -514,7 +507,7 @@ fastExport' repo marks = do
   putStrLn "progress (reading repository)"
   patchset <- readRepo repo
   marksref <- newIORef marks
-  let patches = newset2FL patchset
+  let patches = patchSet2FL patchset
       tags = inOrderTags patchset
       mark :: (PatchInfoAnd rt p) x y -> Int -> TreeIO ()
       mark p n = liftIO $ do putStrLn $ "mark :" ++ show n
@@ -543,7 +536,7 @@ fastExport' repo marks = do
  `finally` do
   putStrLn "progress (cleaning up)"
   current <- readHashedPristineRoot repo
-  cleanHashdir (extractCache repo) HashedPristineDir $ catMaybes [current]
+  cleanHashdir (repoCache repo) HashedPristineDir $ catMaybes [current]
   putStrLn "progress done"
 
 dumpPatches ::  (RepoPatch p, ApplyState p ~ Tree)
@@ -623,7 +616,7 @@ dumpPatch ::  (forall p0 x0 y0 . (PatchInfoAnd rt p0) x0 y0 -> Int -> TreeIO ())
           -> (PatchInfoAnd rt p) x y -> Int
           -> TreeIO ()
 dumpPatch mark p n =
-  do dumpBits [ BLC.pack $ "progress " ++ show n ++ ": " ++ piName (info p)
+  do dumpBits [ BLU.fromString $ "progress " ++ show n ++ ": " ++ piName (info p)
               , "commit refs/heads/master" ]
      mark p n
      dumpBits [ BLU.fromString $ "committer " ++ patchAuthor p ++ " " ++ patchDate p
@@ -722,14 +715,16 @@ instance Show (State p) where
 
 fastImport :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 fastImport _ opts [outrepo] =
-  do createDirectory outrepo
-     withCurrentDirectory outrepo $ do
-       createRepository (patchFormat opts) (withWorkingDir opts) (runPatchIndex opts)
-       withRepoLock NoDryRun (useCache opts) NoUpdateWorking (umask opts) $ RepoJob $ \repo -> do
-         -- TODO implement --dry-run, which would be read-only?
-         marks <- fastImport' repo emptyMarks
-         createPristineDirectoryTree repo "." (withWorkingDir opts)
-         return marks
+  withUMaskFlag (umask ? opts) $ withNewDirectory outrepo $ do
+    EmptyRepository repo <- createRepository
+      (patchFormat ? opts)
+      (withWorkingDir ? opts)
+      (patchIndexNo ? opts)
+      (useCache ? opts)
+    -- TODO implement --dry-run, which would be read-only?
+    marks <- fastImport' repo emptyMarks
+    createPristineDirectoryTree repo "." (withWorkingDir ? opts)
+    return marks
 fastImport _ _ _ = fail "I need exactly one output repository."
 
 fastImport' :: forall rt p r u . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) =>
@@ -749,8 +744,8 @@ fastImport' repo marks = do
         -- sort marks into buckets, since there can be a *lot* of them
         markpath :: Int -> AnchoredPath
         markpath n = floatPath (darcsdir </> "marks")
-                        `appendPath` (Name $ BC.pack $ show (n `div` 1000))
-                        `appendPath` (Name $ BC.pack $ show (n `mod` 1000))
+                        `appendPath` (makeName $ show (n `div` 1000))
+                        `appendPath` (makeName $ show (n `mod` 1000))
 
         makeinfo author message tag = do
           let (name, log) = case BC.unpack message of
@@ -765,16 +760,18 @@ fastImport' repo marks = do
         addtag author msg =
           do info_ <- makeinfo author msg True
              gotany <- liftIO $ doesFileExist $ darcsdir </> "tentative_hashed_pristine"
-             deps <- if gotany then liftIO $ getUncovered `fmap` readTentativeRepo repo
+             deps <- if gotany then liftIO $
+                                      getUncovered `fmap`
+                                        readTentativeRepo repo (repoLocation repo)
                                else return []
-             let ident = NilFL :: FL (RepoPatchV2 Prim) cX cX
+             let ident = NilFL :: FL RepoPatchV2 cX cX
                  patch = NormalP (adddeps (infopatch info_ ident) deps)
-             void $ liftIO $ addToTentativeInventory (extractCache repo)
+             void $ liftIO $ addToTentativeInventory (repoCache repo)
                                                      GzipCompression (n2pia patch)
 
         -- processing items
         updateHashes = do
-          let nodarcs = \(AnchoredPath (Name x:_)) _ -> x /= BC.pack darcsdir
+          let nodarcs = \(AnchoredPath (x:_)) _ -> x /= makeName darcsdir
               hashblobs (File blob@(T.Blob con NoHash)) =
                 do hash <- sha256 `fmap` readBlob blob
                    return $ File (T.Blob con hash)
@@ -809,7 +806,7 @@ fastImport' repo marks = do
 
         process :: State p -> Object -> TreeIO (State p)
         process s (Progress p) = do
-          liftIO $ putStrLn ("progress " ++ BC.unpack p)
+          liftIO $ putStrLn ("progress " ++ decodeLocale p)
           return s
 
         process (Toplevel _ _) End = do
@@ -826,7 +823,7 @@ fastImport' repo marks = do
           if Just what == n
              then addtag author msg
              else liftIO $ putStrLn $ "WARNING: Ignoring out-of-order tag " ++
-                             head (lines $ BC.unpack msg)
+                             head (lines $ decodeLocale msg)
           return (Toplevel n b)
 
         process (Toplevel n _) (Reset branch from) =
@@ -919,7 +916,7 @@ fastImport' repo marks = do
           {- current <- updateHashes -} -- why not?
           (prims :: FL p cX cY)  <- return $ fromPrims $ sortCoalesceFL $ reverseRL ps
           let patch = NormalP (infopatch info_ ((NilFL :: FL p cX cX) +>+ prims))
-          void $ liftIO $ addToTentativeInventory (extractCache repo)
+          void $ liftIO $ addToTentativeInventory (repoCache repo)
                                                   GzipCompression (n2pia patch)
           case mark of
             Nothing -> return ()
