@@ -15,8 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, PatternGuards #-}
-
+{-# LANGUAGE PatternGuards #-}
 
 module Darcs.UI.Commands.Log
     ( changes, log
@@ -26,7 +25,6 @@ module Darcs.UI.Commands.Log
 import Prelude ()
 import Darcs.Prelude
 
-import Unsafe.Coerce (unsafeCoerce)
 import Data.List ( intersect, sort, nub, find )
 import Data.Maybe ( fromMaybe, fromJust, isJust )
 import Control.Arrow ( second )
@@ -36,58 +34,57 @@ import Control.Monad.State.Strict
 import Darcs.UI.PrintPatch ( showFriendly )
 import Darcs.Patch.PatchInfoAnd ( fmapFLPIAP, hopefullyM, info )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias, findRepository )
+import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
-    ( DarcsFlag(GenContext,
-                MachineReadable, Count, Interactive,
-                NumberPatches, XMLOutput, Summary,
-                Verbose, Debug, NoPatchIndexFlag)
-    , doReverse, showChangesOnlyToFiles
-    , useCache, maxCount, umask
-    , verbosity, isUnified, isInteractive, hasSummary
+    ( DarcsFlag
+    , changesReverse, onlyToFiles
+    , useCache, maxCount, hasXmlOutput
+    , verbosity, withContext, isInteractive, verbose
     , fixSubPaths, getRepourl )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Path ( SubPath(), toFilePath,
                     fp2fn, fn2fp, normPath, AbsolutePath, simpleSubPath )
 import Darcs.Repository ( PatchSet, PatchInfoAnd,
-                          withRepositoryDirectory, RepoJob(..),
+                          withRepositoryLocation, RepoJob(..),
                           readRepo, unrecordedChanges,
                           withRepoLockCanFail )
-import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(MyersDiff), UpdateWorking(..) )
+import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(MyersDiff) )
 import Darcs.Util.Lock ( withTempDir )
-import Darcs.Patch.Set ( PatchSet(..), newset2RL )
+import Darcs.Patch.Set ( PatchSet(..), patchSet2RL )
 import Darcs.Patch.Conflict ( Conflict, CommuteNoConflicts )
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.FileHunk ( IsHunk )
-import Darcs.Patch.Info ( toXml, showPatchInfo, escapeXML, PatchInfo )
+import Darcs.Patch.Info ( toXml, toXmlShort, showPatchInfo, displayPatchInfo, escapeXML, PatchInfo )
 import Darcs.Patch.Depends ( findCommonWithThem )
 import Darcs.Patch.Bundle( contextPatches )
 import Darcs.Patch.Prim ( PrimPatchBase )
-import Darcs.Patch.Show ( ShowPatch )
+import Darcs.Patch.Show ( ShowPatch, ShowPatchFor(..) )
 import Darcs.Patch.TouchesFiles ( lookTouch )
 import Darcs.Patch.Type ( PatchType(PatchType) )
 import Darcs.Patch.Apply ( Apply, ApplyState )
 import Darcs.Patch ( IsRepoType, invert, xmlSummary, description,
                      effectOnFilePaths, listTouchedFiles, showPatch )
-import Darcs.Patch.Named.Wrapped ( (:~:)(..) )
+import Darcs.Patch.Named.Wrapped ( (:~:)(..), getdeps )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(NilFL), RL(..), filterOutFLFL, filterRL,
     reverseFL, (:>)(..), mapRL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), unseal2, Sealed(..), seal2 )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Match
     ( MatchFlag
     , firstMatch
     , secondMatch
-    , matchAPatchread
+    , matchAPatch
     , haveNonrangeMatch
     , matchFirstPatchset
     , matchSecondPatchset
     )
 import Darcs.Patch.Matchable ( Matchable )
 import Darcs.Util.Printer ( Doc, simplePrinters, (<+>), prefix, text, vcat,
-                 vsep, (<>), ($$), errorDoc, insertBeforeLastline, empty, RenderMode(..) )
+                 vsep, (<>), ($$), errorDoc, insertBeforeLastline, empty )
 import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Util.Progress ( setProgressMode, debugMessage )
 import Darcs.Util.URL ( isValidLocalPath )
@@ -105,56 +102,6 @@ logHelp = unlines
  ++ logHelp'
  ++ logHelp''
 
-logBasicOpts :: DarcsOption a
-                ([O.MatchFlag]
-                 -> Maybe Int
-                 -> Bool
-                 -> Maybe O.ChangesFormat
-                 -> Maybe O.Summary
-                 -> Bool
-                 -> Maybe String
-                 -> Maybe String
-                 -> Maybe Bool
-                 -> a)
-logBasicOpts
-    = O.matchSeveralOrRange
-    ^ O.matchMaxcount
-    ^ O.onlyToFiles
-    ^ O.changesFormat
-    ^ O.summary
-    ^ O.changesReverse
-    ^ O.possiblyRemoteRepo
-    ^ O.workingRepoDir
-    ^ O.interactive -- False
-
-logAdvancedOpts :: DarcsOption a (O.NetworkOptions -> O.WithPatchIndex -> a)
-logAdvancedOpts = O.network ^ O.patchIndexYes
-
-logOpts :: DarcsOption a
-           ([O.MatchFlag]
-            -> Maybe Int
-            -> Bool
-            -> Maybe O.ChangesFormat
-            -> Maybe O.Summary
-            -> Bool
-            -> Maybe String
-            -> Maybe String
-            -> Maybe Bool
-            -> Maybe O.StdCmdAction
-            -> Bool
-            -> Bool
-            -> O.Verbosity
-            -> Bool
-            -> O.NetworkOptions
-            -> O.WithPatchIndex
-            -> O.UseCache
-            -> Maybe String
-            -> Bool
-            -> Maybe String
-            -> Bool
-            -> a)
-logOpts = logBasicOpts `withStdOpts` logAdvancedOpts
-
 log :: DarcsCommand [DarcsFlag]
 log = DarcsCommand
     { commandProgramName = "darcs"
@@ -163,7 +110,7 @@ log = DarcsCommand
     , commandDescription = "List patches in the repository."
     , commandExtraArgs = -1
     , commandExtraArgHelp = ["[FILE or DIRECTORY]..."]
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = knownFileArgs
     , commandCommand = logCmd
     , commandPrereq = findRepository
     , commandArgdefaults = nodefaults
@@ -173,10 +120,23 @@ log = DarcsCommand
     , commandCheckOptions = ocheck logOpts
     , commandParseOptions = onormalise logOpts
     }
+  where
+    logBasicOpts
+      = O.matchSeveralOrRange
+      ^ O.maxCount
+      ^ O.onlyToFiles
+      ^ O.changesFormat
+      ^ O.summary
+      ^ O.changesReverse
+      ^ O.possiblyRemoteRepo
+      ^ O.repoDir
+      ^ O.interactive
+    logAdvancedOpts = O.network ^ O.patchIndexYes
+    logOpts = logBasicOpts `withStdOpts` logAdvancedOpts
 
 logCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 logCmd fps opts args
-  | GenContext `elem` opts = if not . null $ args
+  | O.changesFormat ? opts == Just O.GenContext = if not . null $ args
       then fail "log --context cannot accept other arguments"
       else logContext opts
   | hasRemoteRepo opts = do
@@ -191,10 +151,10 @@ logCmd fps opts args
       fs <- fixSubPaths fps args
       case fs of
         [] -> putStrLn "No valid arguments were given, nothing to do."
-        _ -> do unless (Interactive `elem` opts)
-                 $ unless (NoPatchIndexFlag `elem` opts)
-                  $ withRepoLockCanFail (useCache opts) YesUpdateWorking (umask opts)
-                    $ RepoJob attemptCreatePatchIndex
+        _ -> do unless (isInteractive False opts)
+                 $ when (O.patchIndexNo ? opts == O.YesPatchIndex)
+                  $ withRepoLockCanFail (useCache ? opts)
+                    $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
                 showLog opts $ Just $ nub $ sort fs
 
 maybeNotNull :: [a] -> Maybe [a]
@@ -215,41 +175,43 @@ remoteSubPaths (arg:args) es = case simpleSubPath arg of
 showLog :: [DarcsFlag] -> Maybe [SubPath] -> IO ()
 showLog opts files =
   let repodir = fromMaybe "." (getRepourl opts) in
-  withRepositoryDirectory (useCache opts) repodir $ RepoJob $ \repository -> do
-  unless (Debug `elem` opts) $ setProgressMode False
+  withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repository -> do
+  unless (O.debug ? opts) $ setProgressMode False
   Sealed unrec <- case files of
     Nothing -> return $ Sealed NilFL
-    Just _ -> Sealed `fmap` unrecordedChanges (UseIndex, ScanKnown, MyersDiff) repository files
+    Just _ -> Sealed `fmap` unrecordedChanges (UseIndex, ScanKnown, MyersDiff)
+                  O.NoLookForMoves O.NoLookForReplaces
+                  repository files
                   `catch` \(_ :: IOException) -> return (Sealed NilFL) -- this is triggered when repository is remote
+  debugMessage "About to read the repository..."
+  patches <- readRepo repository
+  debugMessage "Done reading the repository."
   let normfp = fn2fp . normPath . fp2fn
       undoUnrecordedOnFPs = effectOnFilePaths (invert unrec)
       recFiles = map normfp . undoUnrecordedOnFPs . map toFilePath <$> files
       filtered_changes p =
           maybe_reverse <$>
           getLogInfo
-              (maxCount opts)
+              (maxCount ? opts)
               (parseFlags O.matchSeveralOrRange opts)
-              (showChangesOnlyToFiles opts)
+              (onlyToFiles ? opts)
               recFiles
-              (maybeFilterPatches repository)
+              (maybeFilterPatches repository patches)
               p
-  debugMessage "About to read the repository..."
-  patches <- readRepo repository
-  debugMessage "Done reading the repository."
-  if Interactive `elem` opts
+  if isInteractive False opts
     then do (fp_and_fs, _, _) <- filtered_changes patches
             let fp = map fst fp_and_fs
             viewChanges (logPatchSelOpts opts) fp
-    else do let header = if isJust files && XMLOutput `notElem` opts
+    else do let header = if isJust files && hasXmlOutput opts
                           then text $ "Changes to "++unwords (fromJust recFiles)++":\n"
                           else empty
             debugMessage "About to print the patches..."
-            let printers = if XMLOutput `elem` opts then simplePrinters else fancyPrinters
+            let printers = if hasXmlOutput opts then simplePrinters else fancyPrinters
             ps <- readRepo repository -- read repo again to prevent holding onto
                                        -- values forced by filtered_changes
             logOutput <- changelog opts ps `fmap` filtered_changes patches
-            viewDocWith printers Encode $ header $$ logOutput
-  where maybe_reverse (xs,b,c) = if doReverse opts
+            viewDocWith printers $ header $$ logOutput
+  where maybe_reverse (xs,b,c) = if changesReverse ? opts
                                  then (reverse xs, b, c)
                                  else (xs, b, c)
 
@@ -302,7 +264,7 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
                then matchSecondPatchset matchFlags ps
                else Sealed ps
         pf = if haveNonrangeMatch (PatchType :: PatchType rt p) matchFlags
-             then matchAPatchread matchFlags
+             then matchAPatch matchFlags
              else \_ -> True
 
         filterOutUnrelatedChanges (pfs, renames, doc)
@@ -315,7 +277,7 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
         unrelated fs p
           -- If the change does not affect the patches we are looking at,
           -- we ignore the difference between the two states.
-          | null $ fs `intersect` listTouchedFiles p = unsafeCoerce IsEq
+          | null $ fs `intersect` listTouchedFiles p = unsafeCoerceP IsEq
           | otherwise                                = NotEq
 
 -- | Take a list of filenames and patches and produce a list of patches that
@@ -350,7 +312,7 @@ filterPatchesByNames maxcount fns patches = removeNonRenames $
                 else case hopefullyM hp of
                     Nothing -> do
                         let err = text "Can't find patches prior to:"
-                                  $$ showPatchInfo (info hp)
+                                  $$ displayPatchInfo (info hp)
                         return ([], renames, Just err)
                     Just p ->
                         case lookTouch (Just renames) fs (invert p) of
@@ -382,39 +344,48 @@ changelog :: forall rt p wStart wX
           -> ([(Sealed2 (PatchInfoAnd rt p), [FilePath])], [(FilePath, FilePath)], Maybe Doc)
           -> Doc
 changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
-    | Count `elem` opts = text $ show $ length pis_and_fs
-    | XMLOutput `elem` opts =
+    | O.changesFormat ? opts == Just O.CountPatches = text $ show $ length pis_and_fs
+    | hasXmlOutput opts =
          text "<changelog>"
       $$ vcat created_as_xml
       $$ vcat actual_xml_changes
       $$ text "</changelog>"
-    | Summary `elem` opts || Verbose `elem`  opts =
+    | O.yes (O.summary ? opts) || verbose opts =
         mbAppendErr $ vsep (map (number_patch change_with_summary) pis)
     | otherwise = mbAppendErr $ vsep (map (number_patch description') pis)
     where mbAppendErr = maybe id (\err -> ($$ err)) mbErr
           change_with_summary :: Sealed2 (PatchInfoAnd rt p) -> Doc
           change_with_summary (Sealed2 hp)
-              | Just p <- hopefullyM hp = if MachineReadable `elem` opts
-                                           then showPatch p
-                                           else showFriendly (verbosity opts) (hasSummary O.NoSummary opts) p
-              | otherwise = description hp
-                            $$ indent (text "[this patch is unavailable]")
+            | Just p <- hopefullyM hp =
+              if O.changesFormat ? opts == Just O.MachineReadable
+                then showPatch ForStorage p
+                else showFriendly (verbosity ? opts) (O.summary ? opts) p
+            | otherwise = description hp $$ indent (text "[this patch is unavailable]")
 
-          xml_with_summary (Sealed2 hp)
-              | Just p <- hopefullyM hp = insertBeforeLastline
-                                           (toXml $ info hp) (indent $ xmlSummary p)
+          xml_with_summary (Sealed2 hp) | Just p <- hopefullyM hp =
+                    let
+                      deps = getdeps p
+                      xmlDependencies =
+                        text "<explicit_dependencies>"
+                        $$ vcat (map (indent . toXmlShort) deps)
+                        $$ text "</explicit_dependencies>"
+                      summary | deps == [] = indent $ xmlSummary p
+                              | otherwise = indent $ xmlDependencies $$ xmlSummary p
+                    in
+                      insertBeforeLastline (toXml $ info hp) summary
           xml_with_summary (Sealed2 hp) = toXml (info hp)
           indent = prefix "    "
-          actual_xml_changes = if Summary `elem` opts
-                               then map xml_with_summary pis
-                               else map (toXml . unseal2 info) pis
+          actual_xml_changes =
+            case O.summary ? opts of
+              O.YesSummary -> map xml_with_summary pis
+              O.NoSummary -> map (toXml . unseal2 info) pis
 
           created_as_xml = map create createdAsFs where
             create rename@(_, as) = createdAsXml (first_change_of as) rename
             -- We need to reorder the patches when they haven't been reversed
             -- already, so that we find the *first* patch that modifies a given
             -- file, not the last (by default, the list is oldest->newest).
-            reorderer = if not (doReverse opts) then reverse else id
+            reorderer = if not (changesReverse ? opts) then reverse else id
             oldest_first_pis_and_fs = reorderer pis_and_fs
             couldnt_find fn = error $ "Couldn't find first patch affecting " ++
                                       fn ++ " in pis_and_fs"
@@ -422,13 +393,13 @@ changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
             find_first_change_of fn = fromMaybe (couldnt_find fn)
               (mb_first_change_of fn)
             first_change_of = unseal2 info . fst . find_first_change_of
-          number_patch f x = if NumberPatches `elem` opts
+          number_patch f x = if O.changesFormat ? opts == Just O.NumberPatches
                              then case get_number x of
                                   Just n -> text (show n++":") <+> f x
                                   Nothing -> f x
                              else f x
           get_number :: Sealed2 (PatchInfoAnd re p) -> Maybe Int
-          get_number (Sealed2 y) = gn 1 (newset2RL patchset)
+          get_number (Sealed2 y) = gn 1 (patchSet2RL patchset)
               where iy = info y
                     gn :: Int -> RL (PatchInfoAnd rt p) wStart wY -> Maybe Int
                     gn n (bs:<:b) | seq n (info b) == iy = Just n
@@ -452,12 +423,12 @@ logHelp'' = unlines
 logContext :: [DarcsFlag] -> IO ()
 logContext opts = do
   let repodir = fromMaybe "." $ getRepourl opts
-  withRepositoryDirectory (useCache opts) repodir $ RepoJob $ \repository -> do
+  withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repository -> do
       (_ :> ps') <- contextPatches `fmap` readRepo repository
       let pis = mapRL seal2 ps'
       let header = text "\nContext:\n"
-      let logOutput = maybe (vsep $ map (unseal2 (showPatchInfo.info)) pis) errorDoc Nothing
-      viewDocWith simplePrinters Encode $ header $$ logOutput
+      let logOutput = maybe (vsep $ map (unseal2 (showPatchInfo ForStorage . info)) pis) errorDoc Nothing
+      viewDocWith simplePrinters $ header $$ logOutput
 
 -- | changes is an alias for log
 changes :: DarcsCommand [DarcsFlag]
@@ -475,10 +446,10 @@ createdAsXml pinfo (current, createdAs) =
 
 logPatchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 logPatchSelOpts flags = S.PatchSelectionOptions
-    { S.verbosity = verbosity flags
+    { S.verbosity = verbosity ? flags
     , S.matchFlags = parseFlags O.matchSeveralOrRange flags
     , S.interactive = isInteractive False flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = hasSummary O.NoSummary flags
-    , S.withContext = isUnified flags
+    , S.summary = O.summary ? flags
+    , S.withContext = withContext ? flags
     }
