@@ -30,47 +30,42 @@ import Darcs.Prelude
 import Control.Monad ( forM )
 import Control.Monad.State.Strict( gets )
 
-import qualified Data.ByteString.Char8 as BSC
-import Data.Char ( toLower )
-
-import Darcs.Util.Tree.Monad
-    ( withDirectory, fileExists, directoryExists
-    , virtualTreeMonad, currentDirectory
-    , TreeMonad )
-import qualified Darcs.Util.Tree.Monad as HS ( exists, tree )
+import qualified Darcs.Util.Tree.Monad as TM
+    ( TreeMonad, withDirectory, fileExists, directoryExists
+    , virtualTreeMonad, currentDirectory, exists, tree )
 import Darcs.Util.Tree ( Tree, listImmediate, findTree )
 
 import Darcs.Util.Path
-    ( AnchoredPath(..), Name(..), floatPath )
+    ( AnchoredPath(..), floatPath, eqAnycase )
 
-treeHasAnycase :: (Functor m, Monad m)
+treeHasAnycase :: Monad m
                => Tree m
                -> FilePath
                -> m Bool
 treeHasAnycase tree path =
-    fst `fmap` virtualTreeMonad (existsAnycase $ floatPath path) tree
+    fst `fmap` TM.virtualTreeMonad (existsAnycase $ floatPath path) tree
 
 
-existsAnycase :: (Functor m, Monad m)
+existsAnycase :: Monad m
               => AnchoredPath
-              -> TreeMonad m Bool
+              -> TM.TreeMonad m Bool
 existsAnycase (AnchoredPath []) = return True
-existsAnycase (AnchoredPath (Name x:xs)) = do
-     wd <- currentDirectory
-     Just tree <- gets (flip findTree wd . HS.tree)
-     let subs = [ AnchoredPath [Name n] | (Name n, _) <- listImmediate tree,
-                                          BSC.map toLower n == BSC.map toLower x ]
+existsAnycase (AnchoredPath (x:xs)) = do
+     wd <- TM.currentDirectory
+     Just tree <- gets (flip findTree wd . TM.tree)
+     let subs = [ AnchoredPath [n] | (n, _) <- listImmediate tree,
+                                          eqAnycase n x ]
      or `fmap` forM subs (\path -> do
-       file <- fileExists path
+       file <- TM.fileExists path
        if file then return True
-               else withDirectory path (existsAnycase $ AnchoredPath xs))
+               else TM.withDirectory path (existsAnycase $ AnchoredPath xs))
 
 
-treeHas :: (Functor m, Monad m) => Tree m -> FilePath -> m Bool
-treeHas tree path = fst `fmap` virtualTreeMonad (HS.exists $ floatPath path) tree
+treeHas :: Monad m => Tree m -> FilePath -> m Bool
+treeHas tree path = fst `fmap` TM.virtualTreeMonad (TM.exists $ floatPath path) tree
 
-treeHasDir :: (Functor m, Monad m) => Tree m -> FilePath -> m Bool
-treeHasDir tree path = fst `fmap` virtualTreeMonad (directoryExists $ floatPath path) tree
+treeHasDir :: Monad m => Tree m -> FilePath -> m Bool
+treeHasDir tree path = fst `fmap` TM.virtualTreeMonad (TM.directoryExists $ floatPath path) tree
 
-treeHasFile :: (Functor m, Monad m) => Tree m -> FilePath -> m Bool
-treeHasFile tree path = fst `fmap` virtualTreeMonad (fileExists $ floatPath path) tree
+treeHasFile :: Monad m => Tree m -> FilePath -> m Bool
+treeHasFile tree path = fst `fmap` TM.virtualTreeMonad (TM.fileExists $ floatPath path) tree
