@@ -28,6 +28,9 @@ Notes:
 module Darcs.UI.Options.All
     ( DarcsOption
 
+    -- conversion to 'Bool'
+    , YesNo (..)
+
     -- root
     , RootAction (..)
     , rootActions
@@ -40,6 +43,8 @@ module Darcs.UI.Options.All
     , verbosity
     , timings
     , anyVerbosity
+    , HooksConfig (..) -- re-export
+    , HookConfig (..) -- re-export
     , preHook
     , postHook
     , hooks
@@ -48,7 +53,7 @@ module Darcs.UI.Options.All
 
     -- interactivity
     , XmlOutput (..)
-    , xmloutput
+    , xmlOutput
     , DryRun (..) -- re-export
     , dryRun
     , dryRunXml
@@ -56,23 +61,24 @@ module Darcs.UI.Options.All
     , pipe
     , WantGuiPause (..) -- re-export
     , pauseForGui
-    , askdeps
+    , askDeps
 
     -- patch selection
     , module Darcs.UI.Options.Matching -- re-export
     , SelectDeps (..)
     , selectDeps
     , changesReverse
-    , matchMaxcount
+    , maxCount
 
     -- local or remote repo(s)
     , WorkRepo (..) -- re-export
     , workRepo
-    , workingRepoDir
+    , repoDir
     , RemoteRepos (..) -- re-export
     , remoteRepos
     , possiblyRemoteRepo
     , reponame
+    , NotInRemote (..)
     , notInRemote
     , notInRemoteFlagName
     , RepoCombinator (..)
@@ -80,7 +86,7 @@ module Darcs.UI.Options.All
     , allowUnrelatedRepos
     , justThisRepo
     , WithWorkingDir (..) -- re-export
-    , useWorkingDir
+    , withWorkingDir
     , SetDefault (..) -- re-export
     , setDefault
 
@@ -99,11 +105,13 @@ module Darcs.UI.Options.All
     , LookForMoves (..) -- re-export
     , LookForReplaces (..) -- re-export
     , lookfor
+    , lookforadds
+    , lookforreplaces
+    , lookformoves
 
     -- files to consider
     , UseIndex (..) -- re-export
     , ScanKnown (..) -- re-export
-    , diffing
     , IncludeBoring (..)
     , includeBoring
     , allowProblematicFilenames
@@ -118,7 +126,6 @@ module Darcs.UI.Options.All
     , diffAlgorithm
     , WithContext (..)
     , withContext
-    , unidiff
     , ExternalDiff (..)
     , extDiff
 
@@ -126,7 +133,7 @@ module Darcs.UI.Options.All
     , TestChanges (..)
     , testChanges
     , RunTest (..) -- re-export
-    , test
+    , runTest
     , LeaveTestDir (..) -- re-export
     , leaveTestDir
 
@@ -151,16 +158,17 @@ module Darcs.UI.Options.All
 
     -- merging patches
     , AllowConflicts (..) -- re-export
-    , conflicts
+    , conflictsNo
+    , conflictsYes
     , ExternalMerge (..) -- re-export
-    , useExternalMerge
+    , externalMerge
 
     -- optimizations
     , Compression (..) -- re-export
     , compress
     , usePacks
     , WithPatchIndex (..) -- re-export
-    , patchIndex
+    , patchIndexNo
     , patchIndexYes
     , Reorder (..) -- re-export
     , reorder
@@ -172,6 +180,7 @@ module Darcs.UI.Options.All
     , output
     , Summary (..)
     , summary
+    , maybeSummary
     , RemoteDarcs (..) -- re-export
     , NetworkOptions (..)
     , network
@@ -188,12 +197,11 @@ module Darcs.UI.Options.All
     , selectAuthor
 
     -- annotate
-    , humanReadable
     , machineReadable
 
     -- clone
     , CloneKind (..)
-    , partial
+    , cloneKind
 
     -- dist
     , distname
@@ -225,13 +233,16 @@ module Darcs.UI.Options.All
     , pending
     , nullFlag
 
+    -- show repo
+    , EnumPatches (..)
+    , enumPatches
+
     -- gzcrcs
     , GzcrcsAction (..)
     , gzcrcsActions
 
     -- optimize
     , siblings
-    , reorderPatches
     , optimizePatchIndex
     ) where
 
@@ -241,7 +252,6 @@ import Darcs.Prelude
 import Prelude hiding ( (^) )
 import Data.Char ( isDigit )
 import Data.List ( intercalate )
-import Data.Maybe ( listToMaybe )
 
 import Darcs.Repository.Flags
     ( Compression (..)
@@ -271,6 +281,8 @@ import Darcs.Repository.Flags
     , WithWorkingDir (..)
     , PatchFormat (..)
     , IncludeBoring (..)
+    , HooksConfig (..)
+    , HookConfig (..)
     )
 
 import qualified Darcs.UI.Options.Flags as F ( DarcsFlag(..) )
@@ -288,6 +300,79 @@ type DarcsOption = OptSpec DarcsOptDescr Flag
 
 type RawDarcsOption = forall v. v -> RawOptSpec Flag v
 
+-- * Conversion to 'Bool'
+
+class YesNo a where
+  yes :: a -> Bool
+  no :: a -> Bool
+  no = not . yes
+
+instance YesNo Compression where
+  yes NoCompression = False
+  yes GzipCompression = True
+
+instance YesNo WithPatchIndex where
+  yes NoPatchIndex = False
+  yes YesPatchIndex = True
+
+instance YesNo Reorder where
+  yes NoReorder = False
+  yes Reorder = True
+
+instance YesNo UseCache where
+  yes NoUseCache = False
+  yes YesUseCache = True
+
+instance YesNo DryRun where
+  yes NoDryRun = False
+  yes YesDryRun = True
+
+instance YesNo LookForAdds where
+  yes NoLookForAdds = False
+  yes YesLookForAdds = True
+
+instance YesNo LookForReplaces where
+  yes NoLookForReplaces = False
+  yes YesLookForReplaces = True
+
+instance YesNo LookForMoves where
+  yes NoLookForMoves = False
+  yes YesLookForMoves = True
+
+instance YesNo IncludeBoring where
+  yes NoIncludeBoring = False
+  yes YesIncludeBoring = True
+
+instance YesNo RunTest where
+  yes NoRunTest = False
+  yes YesRunTest = True
+
+instance YesNo SetScriptsExecutable where
+  yes NoSetScriptsExecutable = False
+  yes YesSetScriptsExecutable = True
+
+instance YesNo LeaveTestDir where
+  yes NoLeaveTestDir = False
+  yes YesLeaveTestDir = True
+
+instance YesNo UseIndex where
+  yes IgnoreIndex = False
+  yes UseIndex = True
+
+instance YesNo WantGuiPause where
+  yes NoWantGuiPause = False
+  yes YesWantGuiPause = True
+
+instance YesNo WithWorkingDir where
+  yes NoWorkingDir = False
+  yes WithWorkingDir = True
+
+data EnumPatches = NoEnumPatches | YesEnumPatches deriving (Eq, Show)
+
+instance YesNo EnumPatches where
+  yes NoEnumPatches = False
+  yes YesEnumPatches = True
+
 -- * Root command
 
 -- | Options for darcs iself that act like sub-commands.
@@ -295,13 +380,13 @@ data RootAction = RootHelp | Version | ExactVersion | ListCommands deriving (Eq,
 
 rootActions :: PrimDarcsOption (Maybe RootAction)
 rootActions = withDefault Nothing
-  [ RawNoArg ['h'] ["help", "overview"] F.Help (Just RootHelp)
+  [ RawNoArg ['h'] ["help"] F.Help (Just RootHelp)
     "show a brief description of all darcs commands and top-level options"
-  , RawNoArg ['v'] ["version"] F.Version  (Just Version) "show the darcs version"
+  , RawNoArg ['v','V'] ["version"] F.Version  (Just Version) "show the darcs version"
   , RawNoArg [] ["exact-version"] F.ExactVersion (Just ExactVersion)
     "show the exact darcs version"
     -- the switch --commands is here for compatibility only
-  , RawNoArg [] ["commands","list-options"] F.ListCommands (Just ListCommands)
+  , RawNoArg [] ["commands"] F.ListCommands (Just ListCommands)
     "show plain list of available options and commands, for auto-completion"
   ]
 
@@ -342,14 +427,21 @@ anyVerbosity = debug ^ debugHttp ^ verbosity ^ timings where
 
 -- ** Hooks
 
-hooks :: DarcsOption a (Maybe String -> Bool -> Maybe String -> Bool -> a)
-hooks = preHook ^ postHook
+hooks :: DarcsOption a (HooksConfig -> a)
+hooks = imap (Iso fw bw) $ preHook ^ postHook where
+  fw k (HooksConfig pr po) = k pr po
+  bw k pr po = k (HooksConfig pr po)
 
-preHook :: DarcsOption a (Maybe String -> Bool -> a)
-preHook = prehookCmd ^ hookPrompt "prehook" F.AskPrehook F.RunPrehook
+hookIso :: Iso (Maybe String -> Bool -> a) (HookConfig -> a)
+hookIso = (Iso fw bw) where
+    fw k (HookConfig c p) = k c p
+    bw k c p = k (HookConfig c p)
 
-postHook :: DarcsOption a (Maybe String -> Bool -> a)
-postHook = posthookCmd ^ hookPrompt "posthook" F.AskPosthook F.RunPosthook
+preHook :: DarcsOption a (HookConfig -> a)
+preHook = imap hookIso $ prehookCmd ^ hookPrompt "prehook" F.AskPrehook F.RunPrehook
+
+postHook :: DarcsOption a (HookConfig -> a)
+postHook = imap hookIso $ posthookCmd ^ hookPrompt "posthook" F.AskPosthook F.RunPosthook
 
 prehookCmd :: PrimDarcsOption (Maybe String)
 prehookCmd = withDefault Nothing
@@ -397,30 +489,27 @@ this sub-project (which is already large enough).
 
 data XmlOutput = NoXml | YesXml deriving (Eq, Show)
 
-xmloutput :: PrimDarcsOption XmlOutput
-xmloutput = withDefault NoXml [__xmloutput YesXml]
+instance YesNo XmlOutput where
+  yes NoXml = False
+  yes YesXml = True
+
+xmlOutput :: PrimDarcsOption XmlOutput
+xmlOutput = withDefault NoXml [__xmloutput YesXml]
 
 __xmloutput :: RawDarcsOption
 __xmloutput val = RawNoArg [] ["xml-output"] F.XMLOutput val "generate XML formatted output"
 
--- | NOTE: I'd rather work to have no uses of dryRunNoxml, so that any time
--- --dry-run is a possibility, automated users can examine the results more
--- easily with --xml.
---
--- See also issue2397.
+-- | TODO someone wrote here long ago that any time --dry-run is a possibility
+-- automated users should be able to examine the results more
+-- easily with --xml. See also issue2397.
+-- dryRun w/o xml is currently used in add, pull, and repair.
+
 dryRun :: PrimDarcsOption DryRun
-dryRun = (imap . cps) (Iso fw bw) $ singleNoArg [] ["dry-run"] F.DryRun "don't actually take the action"
-  where
-    fw True = YesDryRun
-    fw False = NoDryRun
-    bw YesDryRun = True
-    bw NoDryRun = False
+dryRun = withDefault NoDryRun
+  [ RawNoArg [] ["dry-run"] F.DryRun YesDryRun "don't actually take the action" ]
 
 dryRunXml :: DarcsOption a (DryRun -> XmlOutput -> a)
-dryRunXml = dryRun ^ xmloutput
-
-__dryrun :: RawDarcsOption
-__dryrun val = RawNoArg [] ["dry-run"] F.DryRun val "don't actually take the action"
+dryRunXml = dryRun ^ xmlOutput
 
 pipe :: PrimDarcsOption Bool
 pipe = singleNoArg [] ["pipe"] F.Pipe "ask user interactively for the patch metadata"
@@ -437,8 +526,8 @@ pauseForGui = withDefault YesWantGuiPause
   , RawNoArg [] ["no-pause-for-gui"] F.NoPauseForGui NoWantGuiPause
     "return immediately after external diff or merge command finishes" ]
 
-askdeps :: PrimDarcsOption Bool
-askdeps = withDefault False
+askDeps :: PrimDarcsOption Bool
+askDeps = withDefault False
   [ RawNoArg [] ["ask-deps"] F.AskDeps True "manually select dependencies"
   , RawNoArg [] ["no-ask-deps"] F.NoAskDeps False "automatically select dependencies" ]
 
@@ -465,22 +554,28 @@ changesReverse = withDefault False
 -- should take either a plain 'String' argument (leaving it to a later stage
 -- to parse the 'String' to an 'Int'), or else a @'Maybe' 'Int'@, taking
 -- the possibility of a failed parse into account.
-matchMaxcount :: PrimDarcsOption (Maybe Int)
-matchMaxcount = OptSpec {..} where
-  ounparse k (Just n) = k [ F.MaxCount n ]
-  ounparse k Nothing = k []
-  oparse k fs = k $ listToMaybe [ s | F.MaxCount s <- fs ]
-  ocheck fs = case [ "--max-count="++show n | F.MaxCount n <- fs ] of
-    cfs@(_:_:_) -> ["conflicting flags: " ++ intercalate ", " cfs]
-    _ -> []
-  odesc = [ strArg [] ["max-count"] (F.MaxCount . toInt) "NUMBER"
-    "return only NUMBER results" ]
-  toInt s = if not (null s) && all isDigit s then read s else (-1)
+maxCount :: PrimDarcsOption (Maybe Int)
+maxCount = (withDefault Nothing
+    [ RawStrArg [] ["max-count"] F.MaxCount unF toV unV "NUMBER"
+      "return only NUMBER results" ])
+    {ocheck=check}
+  where
+    unF f = [ s | F.MaxCount s <- [f] ]
+    unV x = [ show s | Just s <- [x] ]
+    toV s = if good s then Just (read s) else Nothing
+    check fs =
+      [ "invalid argument to --max-count: '"++s++"'" | s <- args, not (good s) ] ++
+      if length args > 1
+        then ["conflicting flags: " ++ intercalate ", " (map ("--max-count="++) args)]
+        else []
+      where
+        args = [ s | F.MaxCount s <- fs ]
+    good s = not (null s) && all isDigit s
 
 -- * Local or remote repo
 
 workRepo :: PrimDarcsOption WorkRepo
-workRepo = imap (Iso fw bw) $ workingRepoDir ^ possiblyRemoteRepo where
+workRepo = imap (Iso fw bw) $ repoDir ^ possiblyRemoteRepo where
   fw k (WorkRepoDir s)         = k (Just s) Nothing
   fw k (WorkRepoPossibleURL s) = k Nothing  (Just s)
   fw k WorkRepoCurrentDir      = k Nothing  Nothing
@@ -488,8 +583,8 @@ workRepo = imap (Iso fw bw) $ workingRepoDir ^ possiblyRemoteRepo where
   bw k Nothing  (Just s)       = k (WorkRepoPossibleURL s)
   bw k Nothing  Nothing        = k WorkRepoCurrentDir
 
-workingRepoDir :: PrimDarcsOption (Maybe String)
-workingRepoDir = singleStrArg [] ["repodir"] F.WorkRepoDir arg "DIRECTORY"
+repoDir :: PrimDarcsOption (Maybe String)
+repoDir = singleStrArg [] ["repodir"] F.WorkRepoDir arg "DIRECTORY"
     "specify the repository directory in which to run"
   where arg (F.WorkRepoDir s) = Just s
         arg _ = Nothing
@@ -518,13 +613,21 @@ remoteRepos = (imap . cps) (Iso fw bw) $ multiStrArg [] ["remote-repo"] F.Remote
 notInRemoteFlagName :: String
 notInRemoteFlagName = "not-in-remote"
 
-notInRemote :: PrimDarcsOption [Maybe String]
-notInRemote =
+data NotInRemote
+  = NotInDefaultRepo
+  | NotInRemotePath String
+
+notInRemote :: PrimDarcsOption [NotInRemote]
+notInRemote = (imap . cps) (Iso (map fw) (map bw)) $
     multiOptStrArg [] [notInRemoteFlagName] F.NotInRemote args "URL/PATH" $
         "select all patches not in the default push/pull repository or at "
         ++ "location URL/PATH"
   where
     args fs = [s | F.NotInRemote s <- fs]
+    fw (Just s) = NotInRemotePath s
+    fw Nothing = NotInDefaultRepo
+    bw (NotInRemotePath s) = Just s
+    bw NotInDefaultRepo = Nothing
 
 data RepoCombinator = Intersection | Union | Complement deriving (Eq, Show)
 
@@ -546,8 +649,8 @@ justThisRepo = singleNoArg [] ["just-this-repo"] F.JustThisRepo
   "Limit the check or repair to the current repo"
 
 -- | convert, clone, init
-useWorkingDir :: PrimDarcsOption WithWorkingDir
-useWorkingDir = withDefault WithWorkingDir
+withWorkingDir :: PrimDarcsOption WithWorkingDir
+withWorkingDir = withDefault WithWorkingDir
   [ RawNoArg [] ["with-working-dir"] F.UseWorkingDir WithWorkingDir
     "Create a working directory (normal repository)"
   , RawNoArg [] ["no-working-dir"] F.UseNoWorkingDir NoWorkingDir
@@ -625,12 +728,12 @@ data LookFor = LookFor
   }
 
 lookfor :: PrimDarcsOption LookFor
-lookfor = imap (Iso fw bw) (lookforadds ^ lookforreplaces ^ lookformoves) where
+lookfor = imap (Iso fw bw) (lookforadds NoLookForAdds ^ lookforreplaces ^ lookformoves) where
   fw k (LookFor a r m) = k a r m
   bw k a r m = k (LookFor a r m)
 
-lookforadds :: PrimDarcsOption LookForAdds
-lookforadds = withDefault NoLookForAdds
+lookforadds :: LookForAdds -> PrimDarcsOption LookForAdds
+lookforadds def = withDefault def
   [ RawNoArg ['l'] ["look-for-adds"] F.LookForAdds YesLookForAdds
     "look for (non-boring) files that could be added"
   , RawNoArg [] ["dont-look-for-adds","no-look-for-adds"] F.NoLookForAdds NoLookForAdds
@@ -654,12 +757,6 @@ lookformoves = withDefault NoLookForMoves
 
 -- * Files to consider
 
-diffing :: PrimDarcsOption (UseIndex, ScanKnown, DiffAlgorithm)
-diffing = imap (Iso curry3 uncurry3) $ useIndex ^ scanKnown ^ diffAlgorithm
-  where
-    uncurry3 k x y z = k (x,y,z)
-    curry3 k (x,y,z) = k x y z
-
 useIndex :: PrimDarcsOption UseIndex
 useIndex = (imap . cps) (Iso fw bw) ignoreTimes where
   fw False = UseIndex
@@ -667,15 +764,6 @@ useIndex = (imap . cps) (Iso fw bw) ignoreTimes where
   bw UseIndex = False
   bw IgnoreIndex = True
 
-scanKnown :: PrimDarcsOption ScanKnown
-scanKnown = imap (Iso fw bw) $ lookforadds ^ includeBoring where
-  fw k ScanKnown = k NoLookForAdds NoIncludeBoring
-  fw k ScanAll = k YesLookForAdds NoIncludeBoring
-  fw k ScanBoring = k YesLookForAdds YesIncludeBoring
-  bw k NoLookForAdds _ = k ScanKnown
-  bw k YesLookForAdds NoIncludeBoring = k ScanAll
-  bw k YesLookForAdds YesIncludeBoring = k ScanBoring
-
 includeBoring :: PrimDarcsOption IncludeBoring
 includeBoring = withDefault NoIncludeBoring
   [ RawNoArg [] ["boring"] F.Boring YesIncludeBoring "don't skip boring files"
@@ -729,6 +817,10 @@ diffAlgorithm = withDefault PatienceDiff
 
 data WithContext = NoContext | YesContext deriving (Eq, Show)
 
+instance YesNo WithContext where
+  yes NoContext = False
+  yes YesContext = True
+
 withContext :: PrimDarcsOption WithContext
 withContext = (imap . cps) (Iso fw bw) $ withDefault False
   [ RawNoArg ['u'] ["unified"] F.Unified True
@@ -740,42 +832,46 @@ withContext = (imap . cps) (Iso fw bw) $ withDefault False
         bw NoContext = False
         bw YesContext = True
 
-unidiff :: PrimDarcsOption Bool
-unidiff = withDefault True
-  [ RawNoArg ['u'] ["unified"] F.Unified True "pass -u option to diff"
-  , RawNoArg  [] ["no-unified"] F.NonUnified False "output patch in diff's dumb format" ]
-
-data ExternalDiff = ExternalDiff { _diffCmd :: Maybe String, _diffOpts :: [String] } deriving (Eq, Show)
+data ExternalDiff = ExternalDiff
+  { diffCmd :: Maybe String
+  , diffOpts :: [String]
+  , diffUnified :: Bool
+  } deriving (Eq, Show)
 
 extDiff :: PrimDarcsOption ExternalDiff
-extDiff = imap (Iso fw bw) $ extDiffCmd ^ extDiffOpts where
-  fw k (ExternalDiff cmd opts) = k cmd opts
-  bw k cmd opts = k (ExternalDiff cmd opts)
+extDiff = imap (Iso fw bw) $ __extDiffCmd ^ __extDiffOpts ^ __unidiff where
+  fw k (ExternalDiff cmd opts uni) = k cmd opts uni
+  bw k cmd opts uni = k (ExternalDiff cmd opts uni)
 
-extDiffCmd :: PrimDarcsOption (Maybe String)
-extDiffCmd = singleStrArg [] ["diff-command"] F.DiffCmd arg "COMMAND"
+__extDiffCmd :: PrimDarcsOption (Maybe String)
+__extDiffCmd = singleStrArg [] ["diff-command"] F.DiffCmd arg "COMMAND"
     "specify diff command (ignores --diff-opts)"
   where arg (F.DiffCmd s) = Just s
         arg _ = Nothing
 
-extDiffOpts :: PrimDarcsOption [String]
-extDiffOpts = multiStrArg [] ["diff-opts"] F.DiffFlags mkV "OPTIONS"
+__extDiffOpts :: PrimDarcsOption [String]
+__extDiffOpts = multiStrArg [] ["diff-opts"] F.DiffFlags mkV "OPTIONS"
     "options to pass to diff"
   where mkV fs = [ s | F.DiffFlags s <- fs ]
 
+__unidiff :: PrimDarcsOption Bool
+__unidiff = withDefault True
+  [ RawNoArg ['u'] ["unified"] F.Unified True "pass -u option to diff"
+  , RawNoArg  [] ["no-unified"] F.NonUnified False "output patch in diff's dumb format" ]
+
 -- * Runnign tests
 
 data TestChanges = NoTestChanges | YesTestChanges LeaveTestDir deriving (Eq)
 
 testChanges :: PrimDarcsOption TestChanges
-testChanges = imap (Iso fw bw) $ test ^ leaveTestDir where
+testChanges = imap (Iso fw bw) $ runTest ^ leaveTestDir where
   fw k NoTestChanges = k NoRunTest {- undefined -} YesLeaveTestDir
   fw k (YesTestChanges ltd) = k YesRunTest ltd
   bw k NoRunTest _ = k NoTestChanges
   bw k YesRunTest ltd = k (YesTestChanges ltd)
 
-test :: PrimDarcsOption RunTest
-test = withDefault NoRunTest
+runTest :: PrimDarcsOption RunTest
+runTest = withDefault NoRunTest
   [ RawNoArg [] ["test"] F.Test YesRunTest "run the test script"
   , RawNoArg [] ["no-test"] F.NoTest NoRunTest "don't run the test script" ]
 
@@ -805,8 +901,8 @@ from = singleStrArg [] ["from"] F.Author arg
         arg _ = Nothing
 
 to :: PrimDarcsOption [String]
-to = multiStrArg [] ["to"] F.Target mkV "EMAIL" "specify destination email"
-  where mkV fs = [ s | F.Target s <- fs ]
+to = multiStrArg [] ["to"] F.To mkV "EMAIL" "specify destination email"
+  where mkV fs = [ s | F.To s <- fs ]
 
 cc :: PrimDarcsOption [String]
 cc = multiStrArg [] ["cc"] F.Cc mkV "EMAIL" "mail results to additional EMAIL(s)"
@@ -937,8 +1033,13 @@ verify = withDefault NoVerify
 
 -- * Merging patches
 
--- applyConflictOptions = conflicts NoAllowConflicts
--- pullConflictOptions = conflicts YesAllowConflictsAndMark
+-- | push, apply, rebase apply: default to 'NoAllowConflicts'
+conflictsNo :: PrimDarcsOption (Maybe AllowConflicts)
+conflictsNo = conflicts NoAllowConflicts
+
+-- | pull, rebase pull: default to 'YesAllowConflictsAndMark'
+conflictsYes :: PrimDarcsOption (Maybe AllowConflicts)
+conflictsYes = conflicts YesAllowConflictsAndMark
 
 conflicts :: AllowConflicts -> PrimDarcsOption (Maybe AllowConflicts)
 conflicts def = withDefault (Just def)
@@ -946,16 +1047,14 @@ conflicts def = withDefault (Just def)
       F.MarkConflicts (Just YesAllowConflictsAndMark) "mark conflicts"
   , RawNoArg [] ["allow-conflicts"]
       F.AllowConflicts (Just YesAllowConflicts) "allow conflicts, but don't mark them"
---   , RawNoArg [] ["no-resolve-conflicts"]
---       NoAllowConflicts "equivalent to --dont-allow-conflicts, for backwards compatibility"
   , RawNoArg [] ["dont-allow-conflicts","no-allow-conflicts","no-resolve-conflicts"]
       F.NoAllowConflicts (Just NoAllowConflicts) "fail if there are patches that would create conflicts"
   , RawNoArg [] ["skip-conflicts"]
       F.SkipConflicts Nothing "filter out any patches that would create conflicts" ]
 
 -- Technically not an isomorphism, see 'sendmailIso'.
-useExternalMerge :: PrimDarcsOption ExternalMerge
-useExternalMerge = imap (Iso fw bw) $ singleStrArg [] ["external-merge"] F.ExternalMerge arg
+externalMerge :: PrimDarcsOption ExternalMerge
+externalMerge = imap (Iso fw bw) $ singleStrArg [] ["external-merge"] F.ExternalMerge arg
     "COMMAND" "use external tool to merge conflicts"
   where
     arg (F.ExternalMerge s) = Just s
@@ -978,8 +1077,8 @@ usePacks = withDefault True
   , RawNoArg [] ["no-packs"] F.NoPacks False "don't use repository packs" ]
 
 -- for init, clone and convert: patch index disabled by default
-patchIndex :: PrimDarcsOption WithPatchIndex
-patchIndex = withDefault NoPatchIndex [__patchIndex YesPatchIndex, __noPatchIndex NoPatchIndex]
+patchIndexNo :: PrimDarcsOption WithPatchIndex
+patchIndexNo = withDefault NoPatchIndex [__patchIndex YesPatchIndex, __noPatchIndex NoPatchIndex]
 
 -- for log and annotate: patch index enabled by default
 patchIndexYes :: PrimDarcsOption WithPatchIndex
@@ -1025,8 +1124,23 @@ output = withDefault Nothing
 
 data Summary = NoSummary | YesSummary deriving (Eq, Show)
 
-summary :: PrimDarcsOption (Maybe Summary)
-summary = withDefault Nothing
+instance YesNo Summary where
+  yes NoSummary = False
+  yes YesSummary = True
+
+-- all commands except whatsnew
+summary :: PrimDarcsOption Summary
+summary = (imap . cps) (Iso fw bw) $ maybeSummary Nothing
+  where
+    fw Nothing = NoSummary
+    fw (Just NoSummary) = NoSummary
+    fw (Just YesSummary) = YesSummary
+    bw NoSummary = Nothing
+    bw YesSummary = Just YesSummary
+
+-- needed for whatsnew
+maybeSummary :: Maybe Summary -> PrimDarcsOption (Maybe Summary)
+maybeSummary def = withDefault def
   [ RawNoArg ['s'] ["summary"] F.Summary (Just YesSummary) "summarize changes"
   , RawNoArg [] ["no-summary"] F.NoSummary (Just NoSummary) "don't summarize changes" ]
 
@@ -1091,24 +1205,21 @@ selectAuthor = singleNoArg [] ["select-author"] F.SelectAuthor
 
 -- ** annotate
 
--- | TODO: These should be mutually exclusive, but are they? The code is almost inscrutable.
-humanReadable :: PrimDarcsOption Bool
-humanReadable = withDefault False [__humanReadable True]
+machineReadable :: PrimDarcsOption Bool
+machineReadable = withDefault False
+  [ __humanReadable False
+  , __machineReadable True ]
 
 __humanReadable :: RawDarcsOption
 __humanReadable val = RawNoArg [] ["human-readable"] F.HumanReadable val "give human-readable output"
 
--- | See above.
-machineReadable :: PrimDarcsOption Bool
-machineReadable = withDefault False [__machineReadable True]
-
 __machineReadable :: RawDarcsOption
 __machineReadable val = RawNoArg [] ["machine-readable"] F.MachineReadable val "give machine-readable output"
 
 -- ** clone
 
-partial :: PrimDarcsOption CloneKind
-partial = withDefault NormalClone
+cloneKind :: PrimDarcsOption CloneKind
+cloneKind = withDefault NormalClone
   [ RawNoArg [] ["lazy"] F.Lazy LazyClone "get patch files only as needed"
   , RawNoArg [] ["complete"] F.Complete CompleteClone "get a complete copy of the repository" ]
 
@@ -1155,11 +1266,18 @@ distzip = singleNoArg [] ["zip"] F.DistZip "generate zip archive instead of gzip
 
 -- ** log
 
-data ChangesFormat = HumanReadable | MachineReadable | GenContext | GenXml | NumberPatches | CountPatches deriving (Eq, Show)
+data ChangesFormat
+  = HumanReadable
+  | MachineReadable
+  | GenContext
+  | GenXml
+  | NumberPatches
+  | CountPatches
+  deriving (Eq, Show)
 
 changesFormat :: PrimDarcsOption (Maybe ChangesFormat)
 changesFormat = withDefault Nothing
-  [ RawNoArg [] ["context"] F.GenContext (Just GenContext) "give output suitable for get --context"
+  [ RawNoArg [] ["context"] F.GenContext (Just GenContext) "give output suitable for clone --context"
   , __xmloutput (Just GenXml)
   , __humanReadable (Just HumanReadable)
   , __machineReadable (Just MachineReadable)
@@ -1191,7 +1309,7 @@ testStrategy = withDefault Once
   , RawNoArg [] ["backoff"] F.Backoff Backoff "exponential backoff search"
   , RawNoArg [] ["bisect"] F.Bisect Bisect "binary instead of linear search" ]
 
--- ** show files/index
+-- ** show files
 
 files :: PrimDarcsOption Bool
 files = withDefault True
@@ -1206,12 +1324,21 @@ directories = withDefault True
 pending :: PrimDarcsOption Bool
 pending = withDefault True
   [ RawNoArg [] ["pending"] F.Pending True "reflect pending patches in output"
-  , RawNoArg [] ["no-pending"] F.NoPending False "only included recorded patches in output" ]
+  , RawNoArg [] ["no-pending"] F.NoPending False "only include recorded patches in output" ]
 
 -- "null" is already taken
 nullFlag :: PrimDarcsOption Bool
 nullFlag = singleNoArg ['0'] ["null"] F.NullFlag "separate file names by NUL characters"
 
+-- ** show repo
+
+enumPatches :: PrimDarcsOption EnumPatches
+enumPatches = withDefault YesEnumPatches
+  [ RawNoArg [] ["enum-patches"] F.EnumPatches YesEnumPatches
+    "include statistics requiring enumeration of patches"
+  , RawNoArg [] ["no-enum-patches"] F.NoEnumPatches NoEnumPatches
+    "don't include statistics requiring enumeration of patches" ]
+
 -- ** gzcrcs
 
 data GzcrcsAction = GzcrcsCheck | GzcrcsRepair deriving (Eq, Show)
@@ -1224,12 +1351,10 @@ gzcrcsActions = withDefault Nothing
 -- ** optimize
 
 siblings :: PrimDarcsOption [AbsolutePath]
-siblings = multiAbsPathArg [] ["sibling"] F.Sibling mkV "URL" "specify a sibling directory"
+siblings = multiAbsPathArg [] ["sibling"] F.Sibling mkV "DIRECTORY"
+    "specify a sibling directory"
   where mkV fs = [ s | F.Sibling s <- fs ]
 
-reorderPatches :: PrimDarcsOption Bool
-reorderPatches = singleNoArg [] ["reorder-patches"] F.Reorder "reorder the patches in the repository"
-
 reorder :: PrimDarcsOption Reorder
 reorder = withDefault NoReorder
   [ RawNoArg [] ["reorder-patches"] F.Reorder Reorder
