@@ -24,12 +24,11 @@ import System.Environment ( getEnvironment )
 import System.Directory ( removeFile, getHomeDirectory,
                           getAppUserDataDirectory, doesDirectoryExist,
                           createDirectory, getDirectoryContents )
-import System.IO.Error ( isDoesNotExistError )
-import System.Posix.Files( getSymbolicLinkStatus, FileStatus, isDirectory
+import System.IO.Error ( isDoesNotExistError, catchIOError )
+import System.Posix.Files( getSymbolicLinkStatus, FileStatus, isDirectory )
 #ifndef WIN32
-                         , setFileMode, ownerModes
+import System.Posix.Files( setFileMode, ownerModes )
 #endif
-                         )
 import System.FilePath.Posix ( (</>) )
 
 import Darcs.Util.Exception ( catchall )
@@ -47,10 +46,9 @@ withCurrentDirectory name m =
         (\oldwd -> setCurrentDirectory oldwd `catchall` return ())
         (const m)
 
-getFileStatus :: FilePath
-              -> IO (Maybe FileStatus)
+getFileStatus :: FilePath -> IO (Maybe FileStatus)
 getFileStatus f =
-  Just `fmap` getSymbolicLinkStatus f `catchall` return Nothing
+  Just `fmap` getSymbolicLinkStatus f `catchIOError` (\_-> return Nothing)
 
 doesDirectoryReallyExist :: FilePath -> IO Bool
 doesDirectoryReallyExist f =
