@@ -63,11 +63,9 @@ import System.IO
     )
 import System.IO.Error
     ( isAlreadyExistsError
-    , alreadyExistsErrorType
     , annotateIOError
-    , mkIOError
+    , catchIOError
     )
-import System.IO.Temp ( withTempDirectory )
 import Control.Exception
     ( IOException
     , bracket
@@ -84,10 +82,9 @@ import System.Directory
     , getDirectoryContents
     , createDirectory
     , getTemporaryDirectory
-    , renameDirectory
+    , removeDirectoryRecursive
     )
 import System.FilePath.Posix ( splitDirectories )
-import qualified System.FilePath as NativeFilePath ( takeDirectory )
 import System.Environment ( lookupEnv )
 
 import Control.Concurrent ( threadDelay )
@@ -413,17 +410,12 @@ addToErrorLoc :: IOException
               -> IOException
 addToErrorLoc ioe s = annotateIOError ioe s Nothing Nothing
 
--- | Do an action in a newly created directory of the given name. The directory
--- is first created with a different temporary name (but under the same parent
--- directory), and becomes renamed to the final name when the action returns
--- without an exception. This operation raises an alreadyExists exception if a
--- file system object of the given name already exists.
+-- | Do an action in a newly created directory of the given name. If the
+-- directory is successfully created but the action raises an exception, the
+-- directory and all its content is deleted. Caught exceptions are re-thrown.
 withNewDirectory :: FilePath -> IO () -> IO ()
 withNewDirectory name action = do
-  exists1 <- doesFileExist name        -- TODO when depending on directory >= 1.2.7
-  exists2 <- doesDirectoryExist name   --      just use doesPathExist
-  when (exists1 || exists2) $
-    throwIO (mkIOError alreadyExistsErrorType name Nothing Nothing)
-  withTempDirectory (NativeFilePath.takeDirectory name) "darcs_new" $ \path -> do
-    withCurrentDirectory path action
-    renameDirectory path name
+  createDirectory name
+  withCurrentDirectory name action `catch` \e -> do
+    removeDirectoryRecursive name `catchIOError` (const $ return ())
+    throwIO (e :: SomeException)
