@@ -8,18 +8,16 @@ import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Util.Printer
-    ( Printer, Printers, Printers'(..), Printable(..), Color(..), RenderMode(..)
+    ( Printer, Printers, Printers'(..), Printable(..), Color(..)
     , invisiblePrinter, (<>), (<?>), Doc(Doc,unDoc), unsafeBothText, simplePrinter, hcat
     , unsafeText, unsafePackedString
     , renderStringWith, prefix
     )
 
-import Control.Monad ( liftM )
-import Control.Exception ( catch, IOException )
 import Debug.Trace ( trace )
 import Data.Char ( isAscii, isPrint, isSpace, isControl, ord, chr )
 import Data.Bits ( bit, xor )
-import System.Environment ( getEnv )
+import System.Environment ( lookupEnv )
 import qualified Data.ByteString.Char8 as BC (unpack, any, last, spanEnd)
 import qualified Data.ByteString       as B (null, init)
 import System.IO.Unsafe ( unsafePerformIO )
@@ -35,20 +33,19 @@ dollar = unsafeBothText "$"
 cr     = unsafeBothText "\r"
 
 errorDoc :: Doc -> a
-errorDoc = error . showDoc Encode
+errorDoc = error . showDoc
 
 traceDoc :: Doc -> a -> a
-traceDoc d = trace (showDoc Encode d)
+traceDoc d = trace (showDoc d)
 
 assertDoc :: Maybe Doc -> a -> a
 assertDoc Nothing x = x
 assertDoc (Just e) _ = errorDoc e
 
-showDoc :: RenderMode -> Doc -> String
+showDoc :: Doc -> String
 showDoc = renderStringWith (fancyPrinters stderr)
 
--- policy
--- | the 'Policy' type is a record containing the variables which control
+-- | The 'Policy' type is a record containing the variables which control
 -- how 'Doc's will be rendered on some output.
 data Policy = Policy { poColor :: Bool    -- ^ overall use of color
                      , poEscape :: Bool   -- ^ overall use of escaping
@@ -74,8 +71,8 @@ getPolicy handle = unsafePerformIO $
 
     envDontEscapeAnything  <- getEnvBool "DARCS_DONT_ESCAPE_ANYTHING"
     envDontEscapeIsprint   <- getEnvBool "DARCS_DONT_ESCAPE_ISPRINT"
-    envUseIsprint          <- getEnvBool "DARCS_USE_ISPRINT" -- for backwards-compatibility
-    envDontEscape8bit      <- getEnvBool "DARCS_DONT_ESCAPE_8BIT"
+    envUseIsprint          <- getEnvBool "DARCS_USE_ISPRINT"
+    envEscape8bit      <- getEnvBool "DARCS_ESCAPE_8BIT"
 
     envDontEscapeExtra  <- getEnvString "DARCS_DONT_ESCAPE_EXTRA"
     envEscapeExtra      <- getEnvString "DARCS_ESCAPE_EXTRA"
@@ -95,7 +92,7 @@ getPolicy handle = unsafePerformIO $
                     poEscape   = not envDontEscapeAnything,
                     poLineColor= doColor && envDoColorLines,
                     poIsprint  = envDontEscapeIsprint || envUseIsprint,
-                    po8bit     = envDontEscape8bit,
+                    po8bit     = not envEscape8bit,
                     poNoEscX   = envDontEscapeExtra,
                     poEscX     = envEscapeExtra,
                     poTrailing = not envDontEscapeTrailingSpace,
@@ -105,9 +102,8 @@ getPolicy handle = unsafePerformIO $
                     poSpace = False
                   }
  where
-  getEnvBool s = (/= "0") `liftM` safeGetEnv s
-  safeGetEnv s = getEnv s `catch` \(_ :: IOException) -> return "0"
-  getEnvString s = getEnv s `catch` \(_ :: IOException) -> return ""
+  getEnvBool s = maybe False (/= "0") <$> lookupEnv s
+  getEnvString s = maybe "" id <$> lookupEnv s
 
 
 {-
@@ -336,47 +332,47 @@ environmentHelpEscapeWhite = ([ "DARCS_DONT_ESCAPE_TRAILING_SPACES",
 
 environmentHelpEscape :: ([String], [String])
 environmentHelpEscape = (["DARCS_DONT_ESCAPE_ANYTHING",
-                          "DARCS_DONT_ESCAPE_ISPRINT",
-                          "DARCS_DONT_ESCAPE_8BIT",
                           "DARCS_DONT_ESCAPE_EXTRA",
-                          "DARCS_ESCAPE_EXTRA"],[
+                          "DARCS_ESCAPE_EXTRA",
+                          "DARCS_DONT_ESCAPE_ISPRINT",
+                          "DARCS_ESCAPE_8BIT"],[
   "Darcs needs to escape certain characters when printing patch contents to",
-  "a terminal. Characters like backspace can otherwise hide patch content",
-  "from the user, and other character sequences can even in some cases",
-  "redirect commands to the shell if the terminal allows it.",
+  "a terminal, depending on the encoding specified in your locale setting.",
   "",
-  "By default darcs will only allow printable 7-bit ASCII",
-  "characters (including space), and the two control characters tab and",
-  "newline. All other octets are printed in quoted form (as `^<control letter>`",
-  "or `\\<hex code>`).",
+  "By default, darcs assumes that your locale encoding is ASCII compatible.",
+  "This includes UTF-8 and some 8-bit encodings like ISO/IEC-8859 (including",
+  "its variants). Since ASCII contains control characters like backspace",
+  "(which could hide patch content from the user when printed literally to",
+  "the terminal), and even ones that may introduce security risks such as",
+  "redirecting commands to the shell, darcs needs to escape such characters.",
+  "They are printed as `^<control letter>` or `\\<hex code>`. Darcs also uses",
+  "special markup for line endings that are preceeded by white space, since",
+  "the white space would otherwise not be recognizable.",
   "",
-  "Darcs has some limited support for locales. If the system's locale is a ",
-  "single-byte character encoding, like the Latin encodings, you can set the",
-  "environment variable DARCS_DONT_ESCAPE_ISPRINT to 1 and darcs will display",
-  "all the printables in the current system locale instead of just the ASCII",
-  "ones. NOTE: This curently does not work on some architectures if darcs",
-  "is compiled with GHC 6.4 or later. Some non-ASCII control characters might",
-  "be printed and can possibly spoof the terminal.",
+  "If you use an encoding that is not ASCII compatible, things are somewhat",
+  "less smooth. Such encodings include UTF-16 and UTF-32, as well as many of",
+  "the encodings that became obsolete with unicode. In this case you have two",
+  "options: you can set DARCS_DONT_ESCAPE_ANYTHING to 1. Then everything that",
+  "doesn't flip code sets should work, and so will all the bells and whistles",
+  "in your terminal. This environment variable can also be handy if you pipe",
+  "the output to a pager or external filter that knows better than darcs how to",
+  "handle your encoding. Note that all escaping, including the special escaping",
+  "of any line ending spaces, will be turned off by this setting.",
   "",
-  "For multi-byte character encodings things are less smooth. UTF-8 will",
-  "work if you set DARCS_DONT_ESCAPE_8BIT to 1, but non-printables outside",
-  "the 7-bit ASCII range are no longer escaped. E.g., the extra control",
-  "characters from Latin-1 might leave your terminal at the mercy of the",
-  "patch contents. Space characters outside the 7-bit ASCII range are no",
-  "longer recognized and will not be properly escaped at line endings.",
+  "Another possibility is to explicitly tell darcs to not escape or escape",
+  "certain bytes, using DARCS_DONT_ESCAPE_EXTRA and DARCS_ESCAPE_EXTRA. Their",
+  "values should be strings consisting of the verbatim bytes in question. The",
+  "do-escapes take precedence over the dont-escapes. Space characters are still",
+  "escaped at line endings though. The special environment variable",
+  "DARCS_DONT_ESCAPE_TRAILING_CR turns off escaping of carriage return last on",
+  "the line (DOS style).",
   "",
-  "As a last resort you can set DARCS_DONT_ESCAPE_ANYTHING to 1. Then",
-  "everything that doesn't flip code sets should work, and so will all the",
-  "bells and whistles in your terminal. This environment variable can also",
-  "be handy if you pipe the output to a pager or external filter that knows",
-  "better than darcs how to handle your encoding. Note that all escaping,",
-  "including the special escaping of any line ending spaces, will be turned",
-  "off by this setting.",
+  "For historical reasons, darcs also supports DARCS_DONT_ESCAPE_ISPRINT and",
+  "DARCS_USE_ISPRINT (which are synonyms). These make sense only for 8-bit",
+  "encodings like ISO-8859 and are no longer needed since nowadays darcs does",
+  "the right thing here by default.",
   "",
-  "There are two environment variables you can set to explicitly tell darcs",
-  "to not escape or escape octets. They are DARCS_DONT_ESCAPE_EXTRA and",
-  "DARCS_ESCAPE_EXTRA. Their values should be strings consisting of the",
-  "verbatim octets in question. The do-escapes take precedence over the",
-  "dont-escapes. Space characters are still escaped at line endings though.",
-  "The special environment variable DARCS_DONT_ESCAPE_TRAILING_CR turns off",
-  "escaping of carriage return last on the line (DOS style)."])
+  "Finally, if you are in a highly security sensitive situation (or just",
+  "paranoid for other reasons), you can set DARCS_ESCAPE_8BIT to 1. This will",
+  "cause darcs to escape every non-ASCII byte in addition to ASCII control",
+  "characters."])
