@@ -2,7 +2,6 @@
 --
 --  BSD3
 {-# LANGUAGE ScopedTypeVariables, MultiParamTypeClasses, FlexibleInstances, BangPatterns #-}
-{-# LANGUAGE CPP #-}
 
 -- | The abstract representation of a Tree and useful abstract utilities to
 -- handle those.
@@ -32,13 +31,15 @@ module Darcs.Util.Tree
     , modifyTree, updateTree, partiallyUpdateTree, updateSubtrees, overlay
     , addMissingHashes ) where
 
+import Prelude ()
+import Darcs.Prelude hiding ( filter )
+
 import Control.Exception( catch, IOException )
-import Prelude hiding( lookup, filter, all, (<$>) )
 import Darcs.Util.Path
 import Darcs.Util.Hash
 
-import qualified Data.ByteString.Lazy.Char8 as BL
-import qualified Data.ByteString.Char8 as BS
+import qualified Data.ByteString.Lazy as BL
+import qualified Data.ByteString as B
 import qualified Data.Map as M
 
 import Data.Maybe( catMaybes, isNothing )
@@ -47,8 +48,6 @@ import Data.List( union, sort )
 import Control.Monad( filterM )
 import Control.Applicative( (<$>) )
 
-#include "impossible.h"
-
 --------------------------------
 -- Tree, Blob and friends
 --
@@ -94,7 +93,7 @@ itemType (File _) = BlobType
 itemType (SubTree _) = TreeType
 itemType (Stub _ _) = TreeType
 
-emptyTree :: (Monad m) => Tree m
+emptyTree :: Tree m
 emptyTree = Tree { items = M.empty
                  , treeHash = NoHash }
 
@@ -104,14 +103,14 @@ emptyBlob = Blob (return BL.empty) NoHash
 makeBlob :: (Monad m) => BL.ByteString -> Blob m
 makeBlob str = Blob (return str) (sha256 str)
 
-makeBlobBS :: (Monad m) => BS.ByteString -> Blob m
+makeBlobBS :: (Monad m) => B.ByteString -> Blob m
 makeBlobBS s' = let s = BL.fromChunks [s'] in Blob (return s) (sha256 s)
 
-makeTree :: (Monad m) => [(Name,TreeItem m)] -> Tree m
+makeTree :: [(Name,TreeItem m)] -> Tree m
 makeTree l = Tree { items = M.fromList l
                   , treeHash = NoHash }
 
-makeTreeWithHash :: (Monad m) => [(Name,TreeItem m)] -> Hash -> Tree m
+makeTreeWithHash :: [(Name,TreeItem m)] -> Hash -> Tree m
 makeTreeWithHash l h = Tree { items = M.fromList l
                             , treeHash = h }
 
@@ -255,7 +254,7 @@ instance (Monad m) => FilterTree Tree m where
 -- | Given two Trees, a @guide@ and a @tree@, produces a new Tree that is a
 -- identical to @tree@, but only has those items that are present in both
 -- @tree@ and @guide@. The @guide@ Tree may not contain any stubs.
-restrict :: (FilterTree t m, Monad n) => Tree n -> t m -> t m
+restrict :: (FilterTree t m) => Tree n -> t m -> t m
 restrict guide tree = filter accept tree
     where accept path item =
               case (find guide path, item) of
@@ -312,7 +311,7 @@ sortedUnion a@(x:xs) b@(y:ys) = case compare x y of
 -- not share any identical subtrees. They are derived from the first and second
 -- parameters respectively and they are always fully expanded. It might be
 -- advantageous to feed the result into 'zipFiles' or 'zipTrees'.
-diffTrees :: forall m. (Functor m, Monad m) => Tree m -> Tree m -> m (Tree m, Tree m)
+diffTrees :: forall m. (Monad m) => Tree m -> Tree m -> m (Tree m, Tree m)
 diffTrees left right =
             if treeHash left `match` treeHash right
                then return (emptyTree, emptyTree)
@@ -398,7 +397,7 @@ modifyTree t_ p_ i_ = snd $ go t_ p_ i_
             bug $ "descending in modifyTree, case = Nothing, path = " ++ show p_
 
 countmap :: forall a k. M.Map k a -> Int
-countmap = M.fold (\_ i -> i + 1) 0
+countmap = M.foldr (\_ i -> i + 1) 0
 
 updateSubtrees :: (Tree m -> Tree m) -> Tree m -> Tree m
 updateSubtrees fun t =
@@ -409,11 +408,11 @@ updateSubtrees fun t =
         update (_, Stub _ _) = bug "Stubs not supported in updateTreePostorder"
 
 -- | Does /not/ expand the tree.
-updateTree :: (Functor m, Monad m) => (TreeItem m -> m (TreeItem m)) -> Tree m -> m (Tree m)
+updateTree :: (Monad m) => (TreeItem m -> m (TreeItem m)) -> Tree m -> m (Tree m)
 updateTree fun t = partiallyUpdateTree fun (\_ _ -> True) t
 
 -- | Does /not/ expand the tree.
-partiallyUpdateTree :: (Functor m, Monad m) => (TreeItem m -> m (TreeItem m))
+partiallyUpdateTree :: (Monad m) => (TreeItem m -> m (TreeItem m))
                        -> (AnchoredPath -> TreeItem m -> Bool) -> Tree m -> m (Tree m)
 partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
   where go path t = do
@@ -433,7 +432,7 @@ partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
 -- object, nor it is allowed for the overlay to add new objects to base.  This
 -- means that the overlay Tree should be a subset of the base Tree (although
 -- any extraneous items will be ignored by the implementation).
-overlay :: (Functor m, Monad m) => Tree m -> Tree m -> Tree m
+overlay :: (Monad m) => Tree m -> Tree m -> Tree m
 overlay base over = Tree { items = M.fromList immediate
                          , treeHash = NoHash }
     where immediate = [ (n, get n) | (n, _) <- listImmediate base ]
@@ -448,7 +447,7 @@ overlay base over = Tree { items = M.fromList immediate
                     (Just x, _) -> x
                     (_, _) -> bug $ "Unexpected case in overlay at get " ++ show n ++ "."
 
-addMissingHashes :: (Monad m, Functor m) => (TreeItem m -> m Hash) -> Tree m -> m (Tree m)
+addMissingHashes :: (Monad m) => (TreeItem m -> m Hash) -> Tree m -> m (Tree m)
 addMissingHashes make = updateTree update -- use partiallyUpdateTree here
     where update (SubTree t) = make (SubTree t) >>= \x -> return $ SubTree (t { treeHash = x })
           update (File blob@(Blob con NoHash)) =
