@@ -26,9 +26,22 @@
 
 . lib
 
-switch_to_utf8_locale
-
 gunzip -c $TESTDATA/metadata-encoding.tgz | tar xf -
-
 cd metadata-encoding
-darcs changes
+
+darcs log -v
+
+switch_to_utf8_locale
+darcs log -v
+darcs clone . ../utf8-clone
+darcs log -v --repodir ../utf8-clone
+
+switch_to_latin9_locale
+darcs log -v
+darcs clone . ../latin-clone
+darcs log -v --repodir ../latin-clone
+
+LC_ALL=C
+darcs log -v
+darcs clone . ../c-clone
+darcs log -v --repodir ../c-clone
