@@ -1,9 +1,7 @@
 #!/usr/bin/env bash
 
-set -ev
 
-# The builtin ! has the wrong semantics for not.
-not () { "$@" && exit 1 || :; }
+. lib
 
 if echo $OS | grep -i windows; then
     echo This test does not work under Windows
